
/*********************************************************/
/*********************************************************/
/***********                                  ************/
/***********  TABLAS DE LA SOLUCION SOFTWARE  ************/
/***********                                  ************/
/*********************************************************/
/*********************************************************/

CREATE TABLE  ESTATS 
   (	ESTAT VARCHAR2(40) ,
	     CONSTRAINT PK_ESTATS PRIMARY KEY (ESTAT) ENABLE
   )
/
CREATE TABLE  PARTITS 
   (	PARTITPOL VARCHAR2(40) ,
	     CONSTRAINT PK_PARTITS PRIMARY KEY (PARTITPOL) ENABLE
   )
/
CREATE TABLE  PARLAMENTS 
   (	NOMPARLAMENT VARCHAR2(40),
      NOMBREPARLAMENTARIS number (15,0) CONSTRAINT NN_NOMBREPARLAMENTARIS NOT NULL ENABLE,
      ESTAT VARCHAR2(40) CONSTRAINT NN_ESTAT NOT NULL ENABLE,
      PAISOREGIO  VARCHAR2(20),
      ADRECA VARCHAR2(100) CONSTRAINT NN_ADRECA NOT NULL ENABLE,
      TELEFON VARCHAR2(40) CONSTRAINT NN_TELEFON NOT NULL ENABLE,
      WEB  VARCHAR2(40),
	     CONSTRAINT PK_PARLAMENTS PRIMARY KEY (NOMPARLAMENT) ENABLE,
       CONSTRAINT FK_PARLAMENTS FOREIGN KEY (ESTAT) REFERENCES ESTATS (ESTAT) ENABLE
   )
/
CREATE TABLE PARLAMENTARIS
(
      NOMPARLAMENT VARCHAR2(40),
      CODIPARLAMENTARI NUMBER (9,0),      
      NIF VARCHAR2(15) CONSTRAINT NN_NIF NOT NULL ENABLE,
      NOM VARCHAR2(40) CONSTRAINT NN_NOM NOT NULL ENABLE,
      COGNOMS VARCHAR2(40) CONSTRAINT NN_COGNOMS NOT NULL ENABLE,
      PARTITPOL VARCHAR2(40),
      CONSTRAINT PK_PARLAMENTARIS PRIMARY KEY (NOMPARLAMENT, CODIPARLAMENTARI ) ENABLE,
      CONSTRAINT FK1_PARLAMENTARIS FOREIGN KEY (PARTITPOL) REFERENCES PARTITS(PARTITPOL) ENABLE,
      CONSTRAINT FK2_PARLAMENTARIS FOREIGN KEY (NOMPARLAMENT) REFERENCES PARLAMENTS (NOMPARLAMENT) ENABLE
)
/

CREATE TABLE ESTATDESPESA
(
      CODIESTAT INTEGER,
      DESCRIPESTAT  VARCHAR2 (40) CONSTRAINT NN_ESTATDESPESA NOT NULL ENABLE,
      CONSTRAINT PK_ESTATDESPESA PRIMARY KEY (CODIESTAT) ENABLE
)
/

CREATE TABLE FORMAPAGAMENT
(
      CODIFORMAPAG INTEGER,
      DESCRIFORMAPAG  VARCHAR2 (40) CONSTRAINT NN_DESCRIFORMAPAG NOT NULL ENABLE,
      CONSTRAINT PK_FORMAPAGAMENT PRIMARY KEY (CODIFORMAPAG) ENABLE
)
/

CREATE TABLE TIPUSDESPESA
(
     TIPUSDESPESA VARCHAR2 (15),--NOMES`POT SER PARLAMENT O PARLAMENTARI
     CODICATDESPESA INTEGER,
     DESCRIPCIODESPESA  VARCHAR2 (50),
     CONSTRAINT PK_TIPUSDESPESA PRIMARY KEY (TIPUSDESPESA, CODICATDESPESA) ENABLE
)
/


CREATE TABLE DESPESES
(
     ID_DESPESA NUMBER (10,0),
     DATA_DESPESA DATE CONSTRAINT NN_DATA_DESPESA NOT NULL ENABLE,
     NOMPARLAMENT VARCHAR2(40) CONSTRAINT NN_NOM_PARLAMENT  NOT NULL ENABLE,
     CODIPARLAMENTARI NUMBER (9,0),
     PARTITPOL VARCHAR2(40), 
     TIPUSDESPESA  VARCHAR2 (15) CONSTRAINT NN_TIPUS_DESPESA NOT NULL ENABLE,
     CODICATDESPESA INTEGER  CONSTRAINT NN_CODICAT_DESPESA NOT NULL ENABLE ,
     IMPORT_DESPESA NUMBER (10,2) CONSTRAINT NN_IMPORT_DESPESA NOT NULL ENABLE ,
     TIPUSIVA NUMBER (10,2),
     NIFPROVEIDOR  VARCHAR2 (15)  ,
     DESCRIPCIO VARCHAR2 (40) CONSTRAINT NN_DESCIPCIO_DESPESA NOT NULL ENABLE ,
     JUSTIFICACIO VARCHAR2 (40) CONSTRAINT  NN_JUSTIFICACIO_DESPESA NOT NULL ENABLE ,
     URLDOCUMENTACIO VARCHAR2 (100) ,
     CODIESTAT INTEGER CONSTRAINT NN_CODIESTAT_DESPESA NOT NULL ENABLE , 
     CODIFORMAPAG INTEGER CONSTRAINT NN_CODIFORMAPAG_DESPESA NOT NULL ENABLE,
     
     CONSTRAINT PK_DESPESES  PRIMARY KEY (ID_DESPESA) ENABLE,
     CONSTRAINT FK1_DESPESES FOREIGN KEY (CODIESTAT)     REFERENCES ESTATDESPESA(CODIESTAT) ENABLE,
     CONSTRAINT FK2_DESPESES FOREIGN KEY (CODIFORMAPAG)  REFERENCES FORMAPAGAMENT(CODIFORMAPAG) ENABLE,  
     CONSTRAINT FK3_DESPESES FOREIGN KEY (TIPUSDESPESA, CODICATDESPESA ) REFERENCES TIPUSDESPESA (TIPUSDESPESA, CODICATDESPESA) ENABLE,
     CONSTRAINT FK4_DESPESES FOREIGN KEY (NOMPARLAMENT) REFERENCES PARLAMENTS(NOMPARLAMENT) ENABLE
)
/

CREATE TABLE LOG_PROCEDIMENTS
(
  ID_LOG              NUMBER NOT NULL,
  DATA_PROCES         VARCHAR2(20)NOT NULL,
  PROC_EXECUTAT       VARCHAR2(50)NOT NULL,
  PARAM_ENTRADA       CLOB,
  PARAM_SORTIDA       CLOB,
  RESULTAT_EXEC       CLOB,
  CONSTRAINT PK_LOG_PROCEDIMENTS PRIMARY KEY (ID_LOG)
);


/********************************************************/
/********************************************************/
/***********                                 ************/
/***********  TABLAS DEL MODULO ESTADISTICO  ************/
/***********                                 ************/
/********************************************************/
/********************************************************/


/* PARA CADA PARLAMENTO: LA SUMA DE TODOS LOS GASTOS EN LOS ULTIMOS 4 AOS */
CREATE TABLE ESTAD_1
( 
  NOMPARLAMENT     VARCHAR2(40),
  IMPORT_DESPESA   NUMBER (10,2),
  PERIODE          VARCHAR2(40),  -- ACTUALIZAREMOS ESTE CAMPO CADA VEZ QUE SE REGISTRE UN GASTO
  CONSTRAINT PK_ESTAD_1  PRIMARY KEY (NOMPARLAMENT) ENABLE
);


/* PARA CADA PARLAMENTO Y AO: DEFERENCIA ENTRE EL PARLAMENTARIO QUE MAS HA GASTADO, Y EL QUE MENOS */
CREATE TABLE ESTAD_2
( 
  NOMPARLAMENT      VARCHAR2(40),
  ANY_ESTAD         INTEGER,
  IMPORT_DIFERENCIA NUMBER (10,2),
  CONSTRAINT PK_ESTAD_2  PRIMARY KEY (NOMPARLAMENT, ANY_ESTAD) ENABLE
);
  
/* PARA CADA AO: EL PARLAMENTO QUE HA TENIDO EL MAXIMO NUMERO DE GASTOS */
CREATE TABLE ESTAD_3
( 
  ANY_ESTAD     INTEGER,  
  NOMPARLAMENT  VARCHAR2(40),
  MAXIM_DESPESA NUMBER (10,2),
  CONSTRAINT PK_ESTAD_3  PRIMARY KEY (ANY_ESTAD) ENABLE
);
  
/* PARA CADA PARLAMENTARIO: SU GASTO TOTAL DE LOS ULTIMOS 4 AOS */
CREATE TABLE ESTAD_4
( 
  CODIPARLAMENTARI      NUMBER (9,0), 
  NOMPARLAMENT          VARCHAR2(40),
  NOM                   VARCHAR2(40),
  COGNOMS               VARCHAR2(40),
  PERIODE               VARCHAR2(40),
  DESPESA_PERIODE       NUMBER (10,2),
  CONSTRAINT PK_ESTAD_4 PRIMARY KEY (CODIPARLAMENTARI, NOMPARLAMENT) ENABLE
);

/* PARA UN AO: LA MEDIA DEL GASTO DE PARLAMENTARIO */
CREATE TABLE ESTAD_5
( 
  ANY_ESTAD             INTEGER,  
  DESPESA_MITJA         NUMBER (10,2),
  CONSTRAINT PK_ESTAD_5 PRIMARY KEY (ANY_ESTAD) ENABLE
);
 
/* PARA UN AO: EL PARLAMENTARIO QUE HA TTENIDO EL MXIMO GASTO, Y EL QUE LO HA TENIDO MINIMO */ 
CREATE TABLE ESTAD_6
( 
  ANY_ESTAD                 INTEGER,
  
  CODIPARLAMENTARI_MAX      NUMBER (9,0), 
  NOMPARLAMENT_MAX          VARCHAR2(40),
  NOM_MAX                   VARCHAR2(40),
  COGNOMS_MAX               VARCHAR2(40),
  DESPESA_MAX               NUMBER (10,2),
  
  CODIPARLAMENTARI_MIN      NUMBER (9,0), 
  NOMPARLAMENT_MIN          VARCHAR2(40),
  NOM_MIN                   VARCHAR2(40),
  COGNOMS_MIN               VARCHAR2(40),
  DESPESA_MIN               NUMBER (10,2),
  
  CONSTRAINT PK_ESTAD_6 PRIMARY KEY (ANY_ESTAD) ENABLE
);

/* PARA UN AO: EL PARLAMENTO CON LA MAYOR MEDIA DE GASTO POR PARLAMENTARIO, AS COMO SU VALOR */
CREATE TABLE ESTAD_7
( 
  NOMPARLAMENT           VARCHAR2(40),
  ANY_ESTAD              INTEGER,
  MITJA_DESPESA          NUMBER (10,2),
  CONSTRAINT PK_ESTAD_7  PRIMARY KEY (ANY_ESTAD) ENABLE
);

/* PARA UN AO: LA SUMA DE TODOS LOS GASTOS DE TODOS LOS PARLAMENTOS */
CREATE TABLE ESTAD_8
( 
  ANY_ESTAD              INTEGER,
  DESPESA                NUMBER (10,2),
  CONSTRAINT PK_ESTAD_8  PRIMARY KEY (ANY_ESTAD) ENABLE
);

CREATE SEQUENCE seq_ID_LOG 
CACHE 10;

CREATE SEQUENCE seq_ID_DESPESA 
CACHE 10;
