-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: A_PARLAMENT 
-- *** REFERENCIA              : A_PARLAMENT 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Inserta nuevos parlamentos
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** ESTADO: Texto
-- *** PAIS/REGION: Texto
-- *** DIRECCION: Texto
-- *** TELEFONO: Texto
-- *** WEB: Texto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_ALTA_PARLAMENT_01: Ya existe el parlamento que se pretende dar de alta
-- *** ERR_ALTA_PARLAMENT_02: El parlamento que se quiere dar de alta es nulo
-- *** ERR_ALTA_PARLAMENT_03: El estado debe existir en la tabla de estados
-- *** ERR_ALTA_PARLAMENT_04: La direccin del parlamento no puede ser nula
-- *** ERR_ALTA_PARLAMENT_05: El telfono del parlamento no puede ser nulo
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE A_PARLAMENT (
             V_NOMPARLAMENT           IN VARCHAR2, 
             V_ESTAT                  IN VARCHAR2,
             V_PAISOREGIO             IN VARCHAR2,
             V_ADRECA                 IN VARCHAR2,
             V_TELEFON                IN VARCHAR2,
             V_WEB                    IN VARCHAR2,
             RSP                      OUT NOCOPY CLOB) AS


    data_proces VARCHAR2(20);
    nom_proces  VARCHAR2(50);
    param_entrada      CLOB;
    parlament_existent  NUMBER ;
    estat_existeix  NUMBER ;

    EXC_PARLAMENT_EXISTEIX     EXCEPTION;--Un parlamento no puede existir con el mismo nombre
    EXC_NOMPARLAMENT_NULL      EXCEPTION;--El nombre del parlamento no puede estar vacio
    EXC_ESTAT_NOEXISTEIX       EXCEPTION;--El estado del parlamento debe existir en la tabla de estados
    EXC_ADRECA_NULL            EXCEPTION;--La direccion no puede ser nula
    EXC_TELEFON_NULL           EXCEPTION;--El telefono no puede ser nulo

    BEGIN
        --DBMS_OUTPUT.put_line(V_NOMPARLAMENT);
        --DBMS_OUTPUT.put_line(V_ESTAT);
        --DBMS_OUTPUT.put_line(V_PAISOREGIO);
        --DBMS_OUTPUT.put_line(V_ADRECA);
        --DBMS_OUTPUT.put_line(V_TELEFON);
        --DBMS_OUTPUT.put_line(V_WEB);

        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='A_PARLAMENT';
        
        --Crea una cadena de texte amb els parmetres de entrada concatenats.
        param_entrada := (
                  'NOMPARLAMENT:'||V_NOMPARLAMENT||'-'||
                  'ESTAT:' ||V_ESTAT||'-'||
                  'PAISOREGIO:'||V_PAISOREGIO||'-'||
                  'ADRECA:'||V_ADRECA||'-'||
                  'TELEFON:'||V_TELEFON||'-'||
                  'WEB:'||V_WEB||'-'        
                  );

       
        -- El parlament que volem donar d'alta, no pot existir
        SELECT COUNT(*) INTO parlament_existent
             FROM PARLAMENTS
             WHERE PARLAMENTS.NOMPARLAMENT = V_NOMPARLAMENT;
        --DBMS_OUTPUT.put_line(parlament_existent);
        
        IF(parlament_existent >0 )
           THEN RAISE EXC_PARLAMENT_EXISTEIX; 
        END IF;
        
        -- EL parlament  que volem donar d'alta, no pot ser  nul
        IF(V_NOMPARLAMENT is NULL)
           THEN RAISE EXC_NOMPARLAMENT_NULL; 
        END IF;        

        -- El estat del parlament ha d'existir
        SELECT COUNT(*) INTO estat_existeix
             FROM ESTATS
             WHERE ESTATS.ESTAT = V_ESTAT;
        --DBMS_OUTPUT.put_line(estat_existeix);
        
        IF(estat_existeix=0)
           THEN RAISE EXC_ESTAT_NOEXISTEIX; 
        END IF;
        
        -- la ADREA que volem donar d'alta, no pot ser  nul
        IF(V_ADRECA is NULL)
           THEN RAISE EXC_ADRECA_NULL; 
        END IF;  
        
        -- eL TELEFON  que volem donar d'alta, no pot ser  nul
        IF(V_TELEFON is NULL)
           THEN RAISE EXC_TELEFON_NULL; 
        END IF; 
        
        -- si arribem fins aqui es que podem donar d'alta el parlament 
        INSERT INTO PARLAMENTS  ("NOMPARLAMENT","NOMBREPARLAMENTARIS","ESTAT","PAISOREGIO","ADRECA","TELEFON","WEB")
              VALUES(V_NOMPARLAMENT,
                     0,
                     V_ESTAT,
                     V_PAISOREGIO,
                     V_ADRECA,
                     V_TELEFON,
                     V_WEB);
                     
        RSP:='OK';
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      RSP);

	--Confirmem la inserci a la Base de Dades:
	COMMIT;
    
  -- Tratamiento de las excepciones
	    
    EXCEPTION 
          WHEN EXC_PARLAMENT_EXISTEIX THEN 
            RSP:='ERR_ALTA_PARLAMENT_01: El parlamento '||V_NOMPARLAMENT||' ya existe';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_NOMPARLAMENT_NULL THEN 
            RSP:='ERR_ALTA_PARLAMENT_02: El parlamento que se quiere dar de alta es nulo';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_ESTAT_NOEXISTEIX THEN 
            RSP:='ERR_ALTA_PARLAMENT_03: El estado  debe existir en la tabla de estados';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_ADRECA_NULL THEN 
            RSP:='ERR_ALTA_PARLAMENT_04: La direccion del parlamento no puede ser nula';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_TELEFON_NULL THEN 
            RSP:='ERR_ALTA_PARLAMENT_05: El telefono del parlamento no puede ser nulo';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;
  
          WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	    COMMIT;

END A_PARLAMENT;
