-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: B_PARLAMENT 
-- *** REFERENCIA              : B_PARLAMENT 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Eliminan parlamentos
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_BAJA_PARLAMENT_01: No existe el parlamento que se quiere dar de baja
-- *** ERR_BAJA_PARLAMENT_02: El parlamento que se quiere dar de baja, tiene parlamentarios
-- *** ERR_BAJA_PARLAMENT_03: El parlamento que se quiere dar de baja, tiene gastos registrados
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE B_PARLAMENT (
             V_NOMPARLAMENT           IN VARCHAR2,
             RSP                      OUT NOCOPY CLOB) AS

    data_proces                VARCHAR2(20);
    nom_proces                 VARCHAR2(50);
    parlament_existent         NUMBER ;
    parlamentari_existent      NUMBER;
    despesaparlament_exisetix  NUMBER;
    param_entrada              CLOB;


    EXC_PARLAMENT_NOEXISTEIX     EXCEPTION;--Un parlamento inexistente
    EXC_PARMANETARI_EXISTEIX     EXCEPTION;--Un parlamento a dar de baja, no puede tener parlamentarios en activo
    EXC_DESPESA_EXISTEIX         EXCEPTION;--Un parlamento no puede tener despesas en activo


    BEGIN
        --DBMS_OUTPUT.put_line(V_NOMPARLAMENT);
        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='B_PARLAMENT';

        param_entrada := ('NOMPARLAMENT:'||V_NOMPARLAMENT);

        -- El parlament que volem donar de baixa, ha d'existir
        SELECT COUNT(*) INTO parlament_existent
             FROM PARLAMENTS
             WHERE PARLAMENTS.NOMPARLAMENT = V_NOMPARLAMENT;
        --DBMS_OUTPUT.put_line(parlament_existent);
        
        IF(parlament_existent =0 )
           THEN RAISE EXC_PARLAMENT_NOEXISTEIX; 
        END IF;

        -- No poden existir parlamentaris d'aquell parlament
        SELECT COUNT(*) INTO parlamentari_existent
             FROM PARLAMENTARIS
             WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT;
        --DBMS_OUTPUT.put_line(parlamentari_existent);
        
        IF(parlamentari_existent>0)
           THEN RAISE EXC_PARMANETARI_EXISTEIX; 
        END IF;

        -- No poden existir despesas d'aquell parlament
        SELECT COUNT(*) INTO despesaparlament_exisetix
             FROM DESPESES
             WHERE DESPESES.NOMPARLAMENT = V_NOMPARLAMENT;

        IF(despesaparlament_exisetix>0)
           THEN RAISE EXC_DESPESA_EXISTEIX; 
        END IF;
        
        -- si arribem fins aqui es que podem donar d'alta el parlament 
	      DELETE FROM PARLAMENTS
        WHERE PARLAMENTS.NOMPARLAMENT=V_NOMPARLAMENT;
        RSP:='OK';
                     
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      RSP);

	--Confirmem la inserci a la Base de Dades:
	COMMIT;
    
  -- Tratamiento de las excepciones
	    
    EXCEPTION 
          WHEN EXC_PARLAMENT_NOEXISTEIX THEN 
            RSP:='ERR_BAJA_PARLAMENT_01: El parlamento '||V_NOMPARLAMENT||' no existe';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

           WHEN EXC_PARMANETARI_EXISTEIX THEN 
            RSP:='ERR_BAJA_PARLAMENT_02: El parlamento '||V_NOMPARLAMENT||' tiene parlamentarios';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

           WHEN EXC_DESPESA_EXISTEIX THEN 
            RSP:='ERR_BAJA_PARLAMENT_03: El parlamento '||V_NOMPARLAMENT||' tiene gastos registrados';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;
          WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	    COMMIT;
 
END B_PARLAMENT;
