-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: M_PARLAMENT 
-- *** REFERENCIA              : M_PARLAMENT 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Modifica datos de los parlamentos
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** PAIS/REGION: Texto
-- *** DIRECCION: Texto
-- *** TELEFONO: Texto
-- *** WEB: Texto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_MOD_PARLAMENT_01: No existe el parlamento que se quiere modificar
-- *** ERR_MOD_PARLAMENT_02: La direccin del parlamento no puede ser nula
-- *** ERR_MOD_PARLAMENT_03: El telfono del parlamento no puede ser nulo
-- ******************************************************************** 
-- COMENTARIOS:
-- NO SE PERMITE MODIFICAR EL ESTADO AL QUE PERTENECE UN PARLAMENTO
-- NO SE PERMITE MODIFICAR EL NUMERO DE PARLAMENTARIOS. ESTOS SE MODIFICAN AUTOMATICAMENTE EN LAS ALTAS O BAJAS DE PARLAMENTARIOS

CREATE OR REPLACE PROCEDURE M_PARLAMENT (
             V_NOMPARLAMENT           IN VARCHAR2,
             V_PAISOREGIO             IN VARCHAR2,
             V_ADRECA                 IN VARCHAR2,
             V_TELEFON                IN VARCHAR2,
             V_WEB                    IN VARCHAR2,
             RSP                      OUT NOCOPY CLOB) AS

    data_proces                VARCHAR2(20);
    nom_proces                 VARCHAR2(50);
    parlament_existent         NUMBER ;
    param_entrada              CLOB;

    EXC_PARLAMENT_NOEXISTEIX     EXCEPTION; --Parlamento inexistente
    EXC_ADRECA_NULL              EXCEPTION; --La direccion no puede ser nula
    EXC_TELEFON_NULL             EXCEPTION; --El telefono no puede ser nulo

    BEGIN
        --DBMS_OUTPUT.put_line(V_NOMPARLAMENT);
        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='M_PARLAMENT';

        --Crea una cadena de texte amb els parmetres de entrada concatenats.
        param_entrada := (
                  'NOMPARLAMENT:'||V_NOMPARLAMENT||'-'||
                  'PAISOREGIO:'||V_PAISOREGIO||'-'||
                  'ADRECA:'||V_ADRECA||'-'||
                  'TELEFON:'||V_TELEFON||'-'||
                  'WEB:'||V_WEB||'-'        
                  );

        -- El parlament que volem modificar, ha d'existir
        SELECT COUNT(*) INTO parlament_existent
             FROM PARLAMENTS
             WHERE PARLAMENTS.NOMPARLAMENT = V_NOMPARLAMENT;
        --DBMS_OUTPUT.put_line(parlament_existent);
        
        IF(parlament_existent =0 )
           THEN RAISE EXC_PARLAMENT_NOEXISTEIX; 
        END IF;
        
        -- la ADREA que volem donar d'alta, no pot ser nul
        IF(V_ADRECA is NULL)
           THEN RAISE EXC_ADRECA_NULL; 
        END IF;  
        
        -- eL TELEFON  que volem donar d'alta, no pot ser  nul
        IF(V_TELEFON is NULL)
           THEN RAISE EXC_TELEFON_NULL; 
        END IF; 
        -- si arribem fins aqui es que podem modificar el parlament 

        UPDATE PARLAMENTS
              SET 
                   PAISOREGIO=V_PAISOREGIO,
                   ADRECA=V_ADRECA,
                   TELEFON=V_TELEFON,
                   WEB=V_WEB
        WHERE  PARLAMENTS.NOMPARLAMENT=V_NOMPARLAMENT;            
        RSP:='OK';
                     
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      RSP);

	--Confirmem la inserci a la Base de Dades:
	COMMIT;
    
  -- Tratamiento de las excepciones
	    
    EXCEPTION 
          WHEN EXC_PARLAMENT_NOEXISTEIX THEN 
            RSP:='ERR_MOD_PARLAMENT_01: El parlamento '||V_NOMPARLAMENT||' no existe';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

         WHEN EXC_ADRECA_NULL THEN 
            RSP:='ERR_MOD_PARLAMENT_02: La direccion del parlamento no puede ser nula';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_TELEFON_NULL THEN 
            RSP:='ERR_MOD_PARLAMENT_03: El telefono del parlamento no puede ser nulo';
            
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;
            
          WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	    COMMIT;

      
END M_PARLAMENT;
