-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: A_PARLAMENTARI 
-- *** REFERENCIA              : A_PARLAMENTARI 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Inserta nuevos parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** CODIGO PARLAMENTARIO: Nmero
-- *** NIF: Texto
-- *** NOMBRE: Texto
-- *** APELLIDOS: Texto
-- *** PARTIDO POLITICO: Texto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_ALTA_PARLAMENTARI_01: El parlamento no existe
-- *** ERR_ALTA_PARLAMENTARI_02: El cdigo del parlamentario ya existe para ese parlamento, o bien es nulo
-- *** ERR_ALTA_PARLAMENTARI_03: El partido poltico no existe
-- *** ERR_ALTA_PARLAMENTARI_04: El NIF del parlamentario no puede ser nulo
-- *** ERR_ALTA_PARLAMENTARI_05: El nombre del parlamentario no puede ser nulo
-- *** ERR_ALTA_PARLAMENTARI_06: Los apellidos del parlamentario no pueden ser nulos
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE A_PARLAMENTARI (
             V_NOMPARLAMENT           IN PARLAMENTARIS.NOMPARLAMENT%type, 
             V_CODIPARLAMENTARI       IN PARLAMENTARIS.CODIPARLAMENTARI%type,
             V_NIF                    IN PARLAMENTARIS.NIF%type,
             V_NOM                    IN PARLAMENTARIS.NOM%type,
             V_COGNOMS                IN PARLAMENTARIS.COGNOMS%type,
             V_PARTITPOL              IN PARLAMENTARIS.PARTITPOL%type,
             RSP                      OUT NOCOPY CLOB) AS
             
    data_proces VARCHAR2(20);
    nom_proces  VARCHAR2(50);
    param_entrada      CLOB;
  

    parlament_existent         NUMBER;-- Variable para controlar si existe el parlamento asociado al parlamentario que se quiere crear    
    parlamentari_existeix      NUMBER;-- Variable PARA CONTROLAR SI UN PARLAMENTARIo ya existe, para el parlamento que se quiere crear
    partitpol_existeix         NUMBER;-- Variable para controlar si existe el partido poltico, del paralemtario que se pretende crear


    EXC_PARLAMENT_INVALID      EXCEPTION;-- El parlamento asociado al parlamentario que se  quiere crear, no existe o es nulo
    EXC_CODIPARLA_INVALID      EXCEPTION;-- El codigo de parlamentario que se quiere crear, ya existe para ese parlemento, o bien es nulo
    EXC_PARTITPOL_INVALID      EXCEPTION;-- El partido politico asociado al parlamentario, no existe o es nulo
    
    EXC_NIF_INVALID            EXCEPTION;--  NIF no informado
    EXC_NOM_INVALID            EXCEPTION;--  Nombre no informado
    EXC_COGNOM_INVALIDD        EXCEPTION;--  Apoellidos no informados
    
Begin

    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='A_PARLAMENTARI';
    
    --Crea una cadena de texte amb els parmetres de entrada concatenats.
    param_entrada := (
              'NOMPARLAMENT:'||V_NOMPARLAMENT||'-'||
              'CODIPARLAMENTARI:' ||V_CODIPARLAMENTARI||'-'||
              'NIF:'||V_NIF||'-'||
              'NOM:'||V_NOM||'-'||
              'COGNOMS:'||V_COGNOMS||'-'||
              'PARTITPOL:'||V_PARTITPOL||'-'        
              );


    -- El parlamento asociado al parlamentario que se  quiere crear, no existe o es nulo
    IF(V_NOMPARLAMENT is NULL) 
        THEN RAISE EXC_PARLAMENT_INVALID; 
    END IF;
        
    SELECT COUNT(*) INTO parlament_existent
         FROM PARLAMENTS
         WHERE PARLAMENTS.NOMPARLAMENT = V_NOMPARLAMENT;
    --DBMS_OUTPUT.put_line(parlament_existent);
    
    IF(parlament_existent =0 )
       THEN RAISE EXC_PARLAMENT_INVALID; 
    END IF;    
    
    --El parlamentario que se quiere crear, ya existe para ese parLamento, o bien es nulo
    IF(V_CODIPARLAMENTARI is NULL) 
        THEN RAISE EXC_CODIPARLA_INVALID; 
    END IF;       
    
    SELECT COUNT (*) INTO parlamentari_existeix
       FROM PARLAMENTARIS
    WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT AND PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
    --DBMS_OUTPUT.put_line('COMPROVACIO PARLAMENTARI'||parlamentari_existeix);  
    IF parlamentari_existeix<>0 
        THEN RAISE EXC_CODIPARLA_INVALID; 
    END IF;
    
    -- El partido politico asociado al parlamentario, no existe o es nulo
    SELECT COUNT(*) INTO partitpol_existeix
         FROM PARTITS
         WHERE PARTITS.PARTITPOL = V_PARTITPOL;
    --DBMS_OUTPUT.put_line(partitpol_existeix);
    
    IF(partitpol_existeix =0 )
       THEN RAISE EXC_PARTITPOL_INVALID; 
    END IF;

   --  NIF no informado
    IF(V_NIF is NULL) 
        THEN RAISE EXC_NIF_INVALID; 
    END IF;
    
   --  Nombre no informado    
    IF(V_NOM is NULL) 
        THEN RAISE EXC_NOM_INVALID; 
    END IF;
    
    --  Apellidos no informados
    IF(V_COGNOMS is NULL) 
        THEN RAISE EXC_COGNOM_INVALIDD; 
    END IF;

    -- Si llego hasta aqui: debo insertar el parlamentario
    INSERT INTO PARLAMENTARIS (NOMPARLAMENT,CODIPARLAMENTARI,NIF,NOM,COGNOMS,PARTITPOL) 
    VALUES (V_NOMPARLAMENT, V_CODIPARLAMENTARI, V_NIF, V_NOM, V_COGNOMS, V_PARTITPOL);
    COMMIT;

    -- iNSERTAM,OS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
    RSP:='OK';
    INSERT INTO LOG_PROCEDIMENTS
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP);
    COMMIT;

    -- Control de las excepciones
    -- El parlamento asociado al parlamentario que se  quiere crear, no existe o es nulo
    EXCEPTION 
        WHEN EXC_PARLAMENT_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_01: El parlamento '||V_NOMPARLAMENT||' No existe';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
    -- El codigo de parlamentario que se quiere crear, ya existe para ese parlemento, o bien es nulo        
        WHEN EXC_CODIPARLA_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_02: El parlamentario '||V_CODIPARLAMENTARI||' ya existe para ese parlamento, o es nulo';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
            
   -- El partido politico asociado al parlamentario, no existe o es nulo   
        WHEN EXC_PARTITPOL_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_03: El partido poltico '||V_PARTITPOL||' No existe';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;


   -- NIF no informado   
        WHEN EXC_NIF_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_04: Falta el NIF del parlamentario que se quiere dar de alta ';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;

   -- NOMBRE no informado   
        WHEN EXC_NOM_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_05: Falta el nombre del parlamentario que se quiere dar de alta';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
   -- APELLIDOS no informados
        WHEN EXC_COGNOM_INVALIDD THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_ALTA_PARLAMENTARI_06: Faltan los apellidos del parlamentario que se quiere dar de alta';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
          
        WHEN OTHERS THEN 
          --DBMS_OUTPUT.PUT_LINE(SQLCODE);
          --DBMS_OUTPUT.PUT_LINE(SQLERRM);
          RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
          INSERT INTO LOG_PROCEDIMENTS
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
       COMMIT;
End A_PARLAMENTARI;



