-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: B_PARLAMENTARI 
-- *** REFERENCIA              : B_PARLAMENTARI 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Elimina parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** CODIGO PARLAMENTARIO: Nmero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_BAJA_PARLAMENTARI_01: No existe el parlamentario que se quiere dar de baja
-- *** ERR_BAJA_PARLAMENTARI_02: El parlamentario tiene gastos asociados
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE B_PARLAMENTARI (
             V_NOMPARLAMENT           IN PARLAMENTARIS.NOMPARLAMENT%type, 
             V_CODIPARLAMENTARI       IN PARLAMENTARIS.CODIPARLAMENTARI%type,
             RSP                      OUT NOCOPY CLOB) AS
             
    data_proces VARCHAR2(20);
    nom_proces  VARCHAR2(50);
    param_entrada      CLOB;

    parlamentari_existeix      NUMBER;-- Variable para controlar si existe el parlamentario que se quiere dar de baja
    despesa_existeix           NUMBER;-- Variable para controlar si existen gastos asociados a ese parlamentario 

    EXC_PARLA_INVALID          exception;-- El parlamentario que se  quiere dar de baja, no existe
    EXC_DESPESA_EXISTEIX       EXCEPTION;-- El parlamentario que se quiere dar de baja, tiene gastos

   
Begin

    DATA_PROCES:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='B_PARLAMENTARI';
    
    --Crea una cadena de texte amb els parmetres de entrada concatenats.
    param_entrada := (
              'NOMPARLAMENT:'||V_NOMPARLAMENT||'-'||
              'CODIPARLAMENTARI:' ||V_CODIPARLAMENTARI     
              );

    -- El parlamentario que se  quiere dar de baja, no existe      
    SELECT COUNT (*) INTO parlamentari_existeix
       FROM PARLAMENTARIS
    WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT AND PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
    --dbms_output.put_line('COMPROVACIO PARLAMENTARI '||parlamentari_existeix);  
    IF parlamentari_existeix=0
        THEN RAISE EXC_PARLA_INVALID; 
    END IF;
    
    -- El parlamentario que se quiere dar de baja, tiene gastos
    SELECT COUNT(*) INTO despesa_existeix
         from DESPESES
         WHERE DESPESES.NOMPARLAMENT = V_NOMPARLAMENT AND DESPESES.CODIPARLAMENTARI = V_CODIPARLAMENTARI ;
    
    IF(despesa_existeix <>0 )
       THEN RAISE EXC_DESPESA_EXISTEIX; 
    END IF;


    -- Si llego hasta aqui: debo borrar el parlamentario
    DELETE FROM  PARLAMENTARIS WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT AND PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
    COMMIT;

    -- INSERTAMOS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
    RSP:='OK';
    INSERT INTO LOG_PROCEDIMENTS
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP);
    COMMIT;

    -- Control de las excepciones
    -- El parlamentario que se  quiere dar de baja, no existe
    EXCEPTION 
        WHEN EXC_PARLA_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_BAJA_PARLAMENTARI_01: El parlamentario numero '||V_CODIPARLAMENTARI||' del parlamento '||'V_NOMPARLAMENT'||' No existe';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
    -- El parlamentario que se quiere dar de baja, tiene gastos      
        WHEN EXC_DESPESA_EXISTEIX THEN 
          RSP:='ERR_BAJA_PARLAMENTARI_02:  El parlamentario numero '||V_CODIPARLAMENTARI||' del parlamento '||'V_NOMPARLAMENT'||' tiene gastos asociados';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;


        WHEN OTHERS THEN 
          --DBMS_OUTPUT.PUT_LINE(SQLCODE);
          --DBMS_OUTPUT.PUT_LINE(SQLERRM);
          RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
          INSERT INTO LOG_PROCEDIMENTS
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
       commit;
End B_PARLAMENTARI;


