-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: M_PARLAMENTARI 
-- *** REFERENCIA              : M_PARLAMENTARI 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Modifica datos de los parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** CODIGO PARLAMENTARIO: Nmero
-- *** NIF: Texto
-- *** NOMBRE: Texto
-- *** APELLIDOS: Texto
-- *** PARTIDO POLITICO: Texto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_MOD_PARLAMENTARI_01: El parlamentario no existe
-- *** ERR_MOD_PARLAMENTARI_02: El partido poltico no existe o es nulo
-- *** ERR_MOD_PARLAMENTARI_03: El NIF del parlamentario no puede ser nulo
-- *** ERR_MOD_PARLAMENTARI_04: El nombre del parlamentario no puede ser nulo
-- *** ERR_MOD_PARLAMENTARI_05: Los apellidos del parlamentario no pueden ser nulos
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE M_PARLAMENTARI (
             V_NOMPARLAMENT           IN PARLAMENTARIS.NOMPARLAMENT%type, 
             V_CODIPARLAMENTARI       IN PARLAMENTARIS.CODIPARLAMENTARI%type,
             V_NIF                    IN PARLAMENTARIS.NIF%type,
             V_NOM                    IN PARLAMENTARIS.NOM%type,
             V_COGNOMS                IN PARLAMENTARIS.COGNOMS%type,
             V_PARTITPOL              IN PARLAMENTARIS.PARTITPOL%type,
             RSP                      OUT NOCOPY CLOB) AS
             
    data_proces VARCHAR2(20);
    nom_proces  VARCHAR2(50);
    param_entrada      CLOB;
    
    parlamentari_existeix      number;-- Variable PARA CONTROLAR SI UN PARLAMENTARIo ya existe, para el parlamento que se quiere crear
    partitpol_existeix         NUMBER;-- Variable para controlar si existe el partido poltico, del paralemtario que se pretende modificar

    EXC_PARLA_INVALID          EXCEPTION;-- El parlamento asociado al parlamentario que se  quiere modificar, no existe o es nulo
    EXC_PARTITPOL_INVALID      EXCEPTION;-- El partido politico asociado al parlamentario, no existe o es nulo
    
    EXC_NIF_INVALID            EXCEPTION;--  NIF no informado
    EXC_NOM_INVALID            exception;--  Nombre no informado
    EXC_COGNOM_INVALID         EXCEPTION;--  Apellidos no informados
    
Begin

    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='M_PARLAMENTARI';
    
    --Crea una cadena de texte amb els parmetres de entrada concatenats.
    param_entrada := (
              'NOMPARLAMENT:'||V_NOMPARLAMENT||'-'||
              'CODIPARLAMENTARI:' ||V_CODIPARLAMENTARI||'-'||
              'NIF:'||V_NIF||'-'||
              'NOM:'||V_NOM||'-'||
              'COGNOMS:'||V_COGNOMS||'-'||
              'PARTITPOL:'||V_PARTITPOL||'-'        
              );

   -- El parlamento asociado al parlamentario que se  quiere modificar, no existe o es nulo    
    SELECT COUNT (*) INTO parlamentari_existeix
       FROM PARLAMENTARIS
    WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT AND PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
    --dbms_output.put_line('COMPROVACIO PARLAMENTARI '||parlamentari_existeix);  
    IF parlamentari_existeix=0
        THEN RAISE EXC_PARLA_INVALID; 
    END IF;
    
    -- El partido politico asociado al parlamentario, no existe o es nulo
    SELECT COUNT(*) INTO partitpol_existeix
         FROM PARTITS
         WHERE PARTITS.PARTITPOL = V_PARTITPOL;
    --DBMS_OUTPUT.put_line(partitpol_existeix);
    
    IF(partitpol_existeix =0 )
       THEN RAISE EXC_PARTITPOL_INVALID; 
    END IF;

   --  NIF no informado
    IF(V_NIF is NULL) 
        THEN RAISE EXC_NIF_INVALID; 
    END IF;
    
   --  Nombre no informado    
    IF(V_NOM is NULL) 
        THEN RAISE EXC_NOM_INVALID; 
    END IF;
    
    --  Apellidos no informados
    IF(V_COGNOMS is NULL) 
        THEN RAISE EXC_COGNOM_INVALID; 
    END IF;

    -- Si llego hasta aqui: debo insertar el parlamentario
    UPDATE PARLAMENTARIS SET NIF=V_NIF, NOM=V_NOM, COGNOMS=V_COGNOMS, PARTITPOL=V_PARTITPOL 
    WHERE  NOMPARLAMENT=V_NOMPARLAMENT AND CODIPARLAMENTARI=V_CODIPARLAMENTARI;
    COMMIT;

    -- iNSERTAM,OS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
    RSP:='OK';
    INSERT INTO LOG_PROCEDIMENTS
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP);
    COMMIT;

    -- Control de las excepciones
    -- El parlamento asociado al parlamentario que se  quiere crear, no existe o es nulo
    EXCEPTION 
        WHEN EXC_PARLA_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_MOD_PARLAMENTARI_01: El parlamentario numero '||V_CODIPARLAMENTARI||' del parlamento '||'V_NOMPARLAMENT'||' No existe';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          

   -- El partido politico asociado al parlamentario, no existe o es nulo   
        when EXC_PARTITPOL_INVALID then --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_MOD_PARLAMENTARI_02: El partido poltico '||V_PARTITPOL||' No existe, o no est informado';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;


   -- NIF no informado   
        WHEN EXC_NIF_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_MOD_PARLAMENTARI_03: Falta el NIF del parlamentario que se quiere dar de alta ';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;

   -- NOMBRE no informado   
        WHEN EXC_NOM_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_MOD_PARLAMENTARI_04: Falta el nombre del parlamentario que se quiere dar de alta';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
   -- APELLIDOS no informados
        WHEN EXC_COGNOM_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
          RSP:='ERR_MOD_PARLAMENTARI_05: Faltan los apellidos del parlamentario que se quiere dar de alta';
          INSERT INTO LOG_PROCEDIMENTS (ID_LOG,DATA_PROCES,PROC_EXECUTAT,PARAM_ENTRADA,PARAM_SORTIDA,RESULTAT_EXEC)
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
          COMMIT;
          
          
        WHEN OTHERS THEN 
          --DBMS_OUTPUT.PUT_LINE(SQLCODE);
          --DBMS_OUTPUT.PUT_LINE(SQLERRM);
          RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
          INSERT INTO LOG_PROCEDIMENTS
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
       COMMIT;
End M_PARLAMENTARI;

