-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: A_DESPESA 
-- *** REFERENCIA              : A_DESPESA 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Inserta registros de gasto de parlamentos o parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** FECHA: Fecha
-- *** PARLAMENTO: Texto
-- *** CODIGO PARLAMENTARIO: Nmero
-- *** TIPO DE GASTO: ("PARLAMENTO", "PARLAMENTARIO")
-- *** CODIGO CAT. DEL GASTO: Nmero
-- *** IMPORTE: Nmero
-- *** IVA: Nmero
-- *** NIF PROVEEDOR: Texto
-- *** DESCRIPCION: Texto
-- *** JUSTIFICACION: Texto
-- *** URL DOCUMENTACION: Texto
-- *** CODIGO DE LA SITUACION EN QUE SE ENCUENTRA EL GASTO: Nmero
-- *** CODIGO DE LA FORMA DE PAGO: Nmero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_ALTA_DESP_01: El parlamento no existe
-- *** ERR_ALTA_DESP_02: El tipo de gasto debe ser *PARLAMENT* o *PARLAMENTARI* obligatoriamente
-- *** ERR_ALTA_DESP_03: Se est  informando un tipo de gasto  *PARLAMENTARI*, pero el cdigo de parlamentario no existe o no est informado
-- *** ERR_ALTA_DESP_04: No existe en cdigo de categora de gasto en la tabla TIPUSDESPESA
-- *** ERR_ALTA_DESP_05: El importe del gasto es nulo
-- *** ERR_ALTA_DESP_06: La descripcin del gasto es nula
-- *** ERR_ALTA_DESP_07: La justificacin del gasto es nula
-- *** ERR_ALTA_DESP_08: No existe el cdigo de gasto en la tabla ESTATDESPESA, o bien es nulo
-- *** ERR_ALTA_DESP_09: No existe el cdigo de la forma de pago en la tabla FORMAPAGAMENT, o bien es nulo
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE A_DESPESA (
           V_DATA_DESPESA              in DESPESES.DATA_DESPESA%type,
           V_NOMPARLAMENT              in DESPESES.NOMPARLAMENT%type,
           V_CODIPARLAMENTARI          in DESPESES.CODIPARLAMENTARI%type,
           
           V_TIPUSDESPESA              in DESPESES.TIPUSDESPESA%type,
           V_CODICATDESPESA            in DESPESES.CODICATDESPESA%type,
           V_IMPORT_DESPESA            in DESPESES.IMPORT_DESPESA%type,
           V_TIPUSIVA                  in DESPESES.TIPUSIVA%type,
           V_NIFPROVEIDOR              in DESPESES.NIFPROVEIDOR%type,
           V_DESCRIPCIO                in DESPESES.DESCRIPCIO%type,
           V_JUSTIFICACIO              in DESPESES.JUSTIFICACIO%type,
           V_URLDOCUMENTACIO           in DESPESES.URLDOCUMENTACIO%type,
           V_CODIESTAT                 in DESPESES.CODIESTAT%type,
           V_CODIFORMAPAG              in DESPESES.CODIFORMAPAG%type,
           RSP                         OUT NOCOPY CLOB) AS

    data_proces VARCHAR2(20);
    nom_proces  VARCHAR2(50);
    param_entrada      CLOB;
   
    RSP_estad                CLOB;--Parmetro para poder invocar las estadisticas   
    V_PARTITPOL              DESPESES.PARTITPOL%type;
    
    parlament_existent       NUMBER;-- Variable para controlar si un parlament existe
    parlamentari_existeix    NUMBER;-- vARIABLE PARA CONTROLAR SI UN PARLAMENTARIO EXISTE PARA UN PArlamento
    codcatdesp_existeix      NUMBER;-- Variable para controlar si el codigo de cat de gasto, existe en la tabla TIPUSDESPESA
    codiestat_existeix       NUMBER;-- Variable para controlar si el "Estado del gasto", existe en la tabla ESTATDESPESA
    codiformapag_existeix    NUMBER;-- Variable para controlar si la forma de pago informada, existe en la tabla  FORMAPAGAMENT
   
    EXC_PARLAMENT_INVALID      EXCEPTION;--Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
    EXC_TIPUSDESP_INVALID      EXCEPTION;--eL TIPUS DE DESPESA tiene que ser obligaroriamente, o de PARLAMENT o de PARLAMENTARI    
    EXC_CODIPARLA_INVALID      EXCEPTION;--El parlamentario para el que se quiere introducir el gasto, no existe para ese parlamento
    EXC_CODCATDES_INVALID      EXCEPTION;--El CODIGO DE CATEGORIA DE DESPESA, no existe en la tabla de TIPUSDESPESA O ES NULO
    EXC_IMPORTDES_INVALID      EXCEPTION;--LA DESPESA no esta informada
    EXC_DESCRIPCI_NULL         EXCEPTION;--La descripcin no est informada
    EXC_JUSTIFICA_NULL         EXCEPTION;--La justificacin no est informada
    EXC_CODIESTAT_INVALID      EXCEPTION;--El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
    EXC_CODIFORMA_INVALID      EXCEPTION;--EL codigode la forma de pago, tiene que existir en la tabla FORMAPAGAMENT

Begin
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_A_DESPESA';
    param_entrada := (
       'DATA_DESPESA:'||V_DATA_DESPESA||'---'||
       'NOMPARLAMENT:'||V_NOMPARLAMENT||'---'||
       'CODIPARLAMENTARI:'||V_CODIPARLAMENTARI||'---'||                           
       'TIPUSDESPESA:'||V_TIPUSDESPESA||'---'||           
       'CODICATDESPESA:'||V_CODICATDESPESA||'---'||          
       'IMPORT_DESPESA:'||V_IMPORT_DESPESA||'---'||         
       'TIPUSIVA:'||V_TIPUSIVA||'---'||                
       'NIFPROVEIDOR:'||V_NIFPROVEIDOR||'---'||           
       'DESCRIPCIO:'||V_DESCRIPCIO||'---'||           
       'JUSTIFICACIO:'||V_JUSTIFICACIO||'---'||            
       'URLDOCUMENTACIO:'||V_URLDOCUMENTACIO||'---'||         
       'CODIESTAT:'||V_CODIESTAT||'---'||                 
       'CODIFORMAPAG:'||V_CODIFORMAPAG
      );


     --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
     IF parlament_existent='' 
       THEN RAISE EXC_PARLAMENT_INVALID; 
     END IF;
      
     SELECT COUNT(*) INTO parlament_existent
       FROM PARLAMENTS
       WHERE PARLAMENTS.NOMPARLAMENT = V_NOMPARLAMENT;
     --DBMS_OUTPUT.put_line(parlament_existent);
      
     IF(parlament_existent = 0 )
       THEN RAISE EXC_PARLAMENT_INVALID; 
     END IF;
    
     --eL TIPUS DE DESPESA tiene que ser obligatroriamente, o de PARLAMENT o de PARLAMENTARI
     IF (V_TIPUSDESPESA<>'PARLAMENT' and V_TIPUSDESPESA<>'PARLAMENTARI' )  
        THEN RAISE EXC_TIPUSDESP_INVALID; 
     END IF;
     
     --El parlamentario para el que se quiere introducir el gasto, no existe para ese parlamento
     if V_TIPUSDESPESA='PARLAMENTARI' then
       SELECT COUNT (*) INTO parlamentari_existeix
         FROM PARLAMENTARIS
      WHERE PARLAMENTARIS.NOMPARLAMENT = V_NOMPARLAMENT AND PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
      --DBMS_OUTPUT.PUT_LINE('COMPROVACIO PARLAMENTARI'||PARLAMENTARI_EXISTEIX);  
        if PARLAMENTARI_EXISTEIX=0 then RAISE EXC_CODIPARLA_INVALID; 
        end if;
      END IF;

      --El CODIGO DE CATEGORIA DE DESPESA, no existe en la tabla de TIPUSDESPESA O ES NULO
      SELECT COUNT (*) INTO codcatdesp_existeix
         FROM TIPUSDESPESA
      WHERE TIPUSDESPESA.CODICATDESPESA = V_CODICATDESPESA AND TIPUSDESPESA.TIPUSDESPESA=V_TIPUSDESPESA;
      --DBMS_OUTPUT.put_line('COMPROVACIO TIPUS DESPESA '||codcatdesp_existeix);  
      IF codcatdesp_existeix=0 THEN RAISE EXC_CODCATDES_INVALID; 
      END IF;

      -- El importe del gasto tiene que tener algun valor
      IF (V_IMPORT_DESPESA=0 OR V_IMPORT_DESPESA IS NULL )
          THEN RAISE EXC_IMPORTDES_INVALID;
      END IF;

      --La descripcin no est informada
      IF V_DESCRIPCIO IS NULL
          THEN RAISE EXC_DESCRIPCI_NULL;
      END IF; 
    
      --La justificacin no est informada
      IF V_JUSTIFICACIO IS NULL
          THEN RAISE EXC_JUSTIFICA_NULL;
      END IF; 
      
      --El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
      SELECT COUNT(*) INTO codiestat_existeix
        FROM ESTATDESPESA
        WHERE ESTATDESPESA.CODIESTAT = V_CODIESTAT;
      --DBMS_OUTPUT.put_line('COMPROVACIO ESTAT DE LA DESPESA '||codiestat_existeix);     
      
      IF(codiestat_existeix = 0 )
        THEN RAISE EXC_CODIESTAT_INVALID; 
      END IF;     
      
      --DBMS_OUTPUT.put_line('FORMA PAGAMENT DE LA DESPESA '||V_CODIFORMAPAG);      
      --EL codigo de la forma de pago, tiene que existir en la tabla FORMAPAGAMENT
      SELECT COUNT(*) INTO codiformapag_existeix
        FROM FORMAPAGAMENT
        WHERE FORMAPAGAMENT.CODIFORMAPAG = V_CODIFORMAPAG;
      --DBMS_OUTPUT.put_line('COMPROVACIO FORMA PAGAMENT DE LA DESPESA '||codiformapag_existeix);     
      
      IF(codiformapag_existeix = 0 )
        THEN RAISE EXC_CODIFORMA_INVALID; 
      END IF;  
    

      -- SI LLEGAMOS HASTA AQUI, PUEDO INSERTAR EL GASTO
      -- SI SE TRATA DE UNA DESPESA DE PARLAMENTARIO
      -- AVERUGUO EL PARTIDO POLITICO AL QUE PERTENECE
      IF V_TIPUSDESPESA='PARLAMENTARI' THEN
         SELECT PARLAMENTARIS.PARTITPOL INTO V_PARTITPOL FROM PARLAMENTARIS WHERE
         PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI AND
         PARLAMENTARIS.NOMPARLAMENT=V_NOMPARLAMENT;
         ELSE V_PARTITPOL:=NULL;
      END IF;
               
      INSERT INTO DESPESES (
          ID_DESPESA,
          DATA_DESPESA,
          NOMPARLAMENT,
          CODIPARLAMENTARI,
          PARTITPOL,
          TIPUSDESPESA,
          CODICATDESPESA,
          IMPORT_DESPESA,
          TIPUSIVA,
          NIFPROVEIDOR,
          DESCRIPCIO,
          JUSTIFICACIO,
          URLDOCUMENTACIO,
          CODIESTAT,
          CODIFORMAPAG) 
        VALUES  (
          seq_ID_DESPESA.Nextval,
          V_DATA_DESPESA,
          V_NOMPARLAMENT,
          V_CODIPARLAMENTARI,
          V_PARTITPOL,
          V_TIPUSDESPESA,
          V_CODICATDESPESA,
          V_IMPORT_DESPESA,
          V_TIPUSIVA,
          V_NIFPROVEIDOR,
          V_DESCRIPCIO,
          V_JUSTIFICACIO,
          V_URLDOCUMENTACIO,
          V_CODIESTAT,
          V_CODIFORMAPAG
          );

      COMMIT;
 
      -- INSERTAMOS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
      RSP:='OK';
      INSERT INTO LOG_PROCEDIMENTS
            VALUES (SEQ_ID_LOG.NextVal,
                    data_proces,
                    nom_proces,
                    param_entrada,
                    NULL,
                    RSP);
      COMMIT;
   
    /*******************************************/
    /*** aqui: llamar al procedimiento       ***/
    /*** para RECALCULAR LAS ESTADISTICAS !! ***/
    /*******************************************/                      
    ESTADISTIQUES (V_DATA_DESPESA,V_NOMPARLAMENT,V_CODIPARLAMENTARI,V_IMPORT_DESPESA, RSP_estad);
   
  
    EXCEPTION 
          WHEN EXC_PARLAMENT_INVALID THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
            RSP:='ERR_ALTA_DESP_01: El parlamento '||V_NOMPARLAMENT||' No existe';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

          WHEN EXC_TIPUSDESP_INVALID THEN --eL TIPUS DE DESPESA tiene que ser obligatroriamente, o de PARLAMENT o de PARLAMENTARI
            RSP:='ERR_ALTA_DESP_02: El TIPO DE GASTO DEBE SER OBLIGATORIAMENTE *PARLAMENT* o *PARLAMENTARI*';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);          
          COMMIT;
          
          WHEN EXC_CODIPARLA_INVALID THEN --El parlamentario para el que se quiere introducir el gasto, no existe para ese parlamento
            RSP:='ERR_ALTA_DESP_03: El parlamentario '||V_CODIPARLAMENTARI||' No existe para el parlamento '||V_NOMPARLAMENT;
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);            
          COMMIT;
          
          WHEN EXC_CODCATDES_INVALID THEN --El CODIGO DE CATEGORIA DE DESPESA, no existe en la tabla de TIPUSDESPESA O ES NULO
            RSP:='ERR_ALTA_DESP_04: El codigo de cat. de gasto '||V_CODICATDESPESA||' No existe en la tabla de gastos en la categoria: '||V_TIPUSDESPESA;
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
          COMMIT;
                
          
          WHEN EXC_IMPORTDES_INVALID THEN --LA DESPESA no esta informada
            RSP:='ERR_ALTA_DESP_05: Importe de gasto no informado';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);          
          COMMIT;
          
          WHEN EXC_DESCRIPCI_NULL THEN --La descripcin no est informada
            RSP:='ERR_ALTA_DESP_06: Descripcin del gasto no informada ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);           
          COMMIT;          
 
          WHEN EXC_JUSTIFICA_NULL THEN --La justificacin no est informada
            RSP:='ERR_ALTA_DESP_07: Justificacin del gasto no informada ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);            
          COMMIT;         

          WHEN EXC_CODIESTAT_INVALID THEN --El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
            RSP:='ERR_ALTA_DESP_08: Codigo de estado de gasto: '||V_CODIESTAT||' invalido o no existente ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);          
          COMMIT;  
          
          WHEN EXC_CODIFORMA_INVALID THEN --EL codigode la forma de pago, tiene que existir en la tabla FORMAPAGAMENT
            RSP:='ERR_ALTA_DESP_09: Codigo de la forma de pago: '||V_CODIFORMAPAG||' invalida o no existente ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
          COMMIT;       
          
          WHEN OTHERS THEN 

            --DBMS_OUTPUT.PUT_LINE(SQLCODE);
            --DBMS_OUTPUT.PUT_LINE(SQLERRM);
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	       COMMIT;

END A_DESPESA;
