-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: B_DESPESA 
-- *** REFERENCIA              : B_DESPESA 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Ellimina registros de gasto de parlamentos o parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** ID DEL GASTO: Nmero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_BAJA_DESP_01: No existe el gasto que se quiere dar de baja
-- ******************************************************************** 

CREATE OR REPLACE PROCEDURE B_DESPESA (
           V_ID_DESPESA                in DESPESES.id_DESPESA%type,
           RSP                         OUT NOCOPY CLOB) AS
           

  data_proces                VARCHAR2(20);
  nom_proces                 VARCHAR2(50);
  param_entrada              CLOB;

  RSP_estad                  CLOB;--Parmetro para poder invocar las estadisticas 
  
  -- Variables para tomar nota de los datos del gasto que vamos a borrar
  -- Y de este modo poder posteriormente recalcular las estadisticas
  V_DATA_DESPESA               DESPESES.DATA_DESPESA%type;
  V_NOMPARLAMENT               DESPESES.NOMPARLAMENT%type;
  V_CODIPARLAMENTARI           DESPESES.CODIPARLAMENTARI%type;
  V_IMPORT_DESPESA             DESPESES.IMPORT_DESPESA%type;

  despesa_existeix           NUMBER;
  EXC_DESPESA_NOEXISTEIX     EXCEPTION;--El Gasto debe existir
  
BEGIN

    /* En principio un gasto siempre se puede borrar */
    /* Solamente sera necesario recalcular las estadisticas */
    
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_B_DESPESA';
    param_entrada := 'ID_DESPESA:'||V_ID_DESPESA;

    SELECT COUNT (*) INTO despesa_existeix from DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA;
    
    IF despesa_existeix=0
       THEN RAISE EXC_DESPESA_NOEXISTEIX; 
    END IF;
    

    /* Podemos proceder a modificar el gasto */
    /* Procedemos a guardarnos sus valores porque despues de borrarlo, tendemos que recalculas las estadicticas */
    
    Select DESPESES.DATA_DESPESA, 
           DESPESES.NOMPARLAMENT, 
           DESPESES.CODIPARLAMENTARI, 
           DESPESES.IMPORT_DESPESA

      INTO V_DATA_DESPESA, 
           V_NOMPARLAMENT,
           V_CODIPARLAMENTARI,
           V_IMPORT_DESPESA
    FROM DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA; 
    
    /* A continuacin borramos el gasto */
    DELETE FROM DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA;


    /*******************************************/
    /*** aqui: llamar al procedimiento       ***/
    /*** para RECALCULAR LAS ESTADISTICAS !! ***/
    /*******************************************/                     
    ESTADISTIQUES (V_DATA_DESPESA,V_NOMPARLAMENT,V_CODIPARLAMENTARI,-V_IMPORT_DESPESA, RSP_estad); 
    
    -- INSERTAMOS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
    RSP:='OK';
    INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP);
    COMMIT;
    
    EXCEPTION 
       WHEN EXC_DESPESA_NOEXISTEIX THEN --Un parlamento para que se quiere introducir el gasto, no existe O ES NULO
            RSP:='ERR_BAJA_DESP_01: El gasto '||V_ID_DESPESA||' No existe';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;

       WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	    COMMIT;
            
end B_DESPESA;

