-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: M_DESPESA 
-- *** REFERENCIA              : M_DESPESA 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Modifica registros de gasto de parlamentos o parlamentarios
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** ID DEL GASTO: Nmero
-- *** IMPORTE: Nmero
-- *** IVA: Nmero
-- *** NIF PROVEEDOR: Texto
-- *** DESCRIPCION: Texto
-- *** JUSTIFICACION: Texto
-- *** URL DOCUMENTACION: Texto
-- *** CODIGO DE LA SITUACION EN QUE SE ENCUENTRA EL GASTO: Nmero
-- *** CODIGO DE LA FORMA DE PAGO: Nmero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERR_MOD_DESP_01: El gasto no existe
-- *** ERR_MOD_DESP_02: Importe del gasto no informado
-- *** ERR_MOD_DESP_03: Descripcin del gasto no informada
-- *** ERR_MOD_DESP_04: Justificacin del gasto no informada
-- *** ERR_MOD_DESP_05: No existe el cdd. de gasto en la tabla ESTATDESPESA, o bien es nulo
-- *** ERR_MOD_DESP_06: No existe el cdigo de la forma de pago en la tabla FORMAPAGAMENT, o bien es nulo
-- ******************************************************************** 
    
CREATE OR REPLACE PROCEDURE M_DESPESA (
           V_ID_DESPESA                in DESPESES.Id_DESPESA%type,
           V_IMPORT_DESPESA            in DESPESES.IMPORT_DESPESA%type,
           V_TIPUSIVA                  in DESPESES.TIPUSIVA%type,
           V_NIFPROVEIDOR              in DESPESES.NIFPROVEIDOR%type,
           V_DESCRIPCIO                in DESPESES.DESCRIPCIO%type,
           V_JUSTIFICACIO              in DESPESES.JUSTIFICACIO%type,
           V_URLDOCUMENTACIO           in DESPESES.URLDOCUMENTACIO%type,
           V_CODIESTAT                 in DESPESES.CODIESTAT%type,
           V_CODIFORMAPAG              in DESPESES.CODIFORMAPAG%type,
           RSP                         OUT NOCOPY CLOB) AS
           
  data_proces                VARCHAR2(20);
  nom_proces                 VARCHAR2(50);

  PARAM_ENTRADA              clob;
  RSP_estad                  CLOB;--Parmetro para poder invocar las estadisticas  
  
  -- Variables para tomar nota de los datos del gasto que vamos a borrar
  -- Y de este modo poder posteriormente recalcular las estadisticas
  VM_DATA_DESPESA               DESPESES.DATA_DESPESA%type;
  VM_NOMPARLAMENT               DESPESES.NOMPARLAMENT%type;
  VM_CODIPARLAMENTARI           DESPESES.CODIPARLAMENTARI%type;
  VM_IMPORT_DESPESA             DESPESES.IMPORT_DESPESA%type;

  -- VARIABLES PARA CONTROL DE LAS EXCEPCIONES
  despesa_existeix           NUMBER;
  codiestat_existeix         NUMBER;-- Variable para controlar si el "Estado del gasto", existe en la tabla ESTATDESPESA
  codiformapag_existeix      NUMBER;-- Variable para controlar si la forma de pago informada, existe en la tabla  FORMAPAGAMENT
  
  EXC_DESPESA_NOEXISTEIX     EXCEPTION;--El Gasto debe existir
  EXC_IMPORTDES_INVALID      EXCEPTION;--LA DESPESA no esta informada
  EXC_DESCRIPCI_NULL         EXCEPTION;--La descripcin no est informada
  EXC_JUSTIFICA_NULL         EXCEPTION;--La justificacin no est informada
  EXC_CODIESTAT_INVALID      EXCEPTION;--El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
  EXC_CODIFORMA_INVALID      EXCEPTION;--EL codigo de la forma de pago, tiene que existir en la tabla FORMAPAGAMENT

  
BEGIN

    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_M_DESPESA';
        param_entrada := (
           'ID_DESPESA:'||V_ID_DESPESA||'---'||                       
           'IMPORT_DESPESA:'||V_IMPORT_DESPESA||'---'||         
           'TIPUSIVA:'||V_TIPUSIVA||'---'||                
           'NIFPROVEIDOR:'||V_NIFPROVEIDOR||'---'||           
           'DESCRIPCIO:'||V_DESCRIPCIO||'---'||           
           'JUSTIFICACIO:'||V_JUSTIFICACIO||'---'||            
           'URLDOCUMENTACIO:'||V_URLDOCUMENTACIO||'---'||         
           'CODIESTAT:'||V_CODIESTAT||'---'||                 
           'CODIFORMAPAG:'||V_CODIFORMAPAG
          );

    -- Control de error: el gasto debe existir
    SELECT COUNT (*) INTO despesa_existeix from DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA;   
    IF despesa_existeix=0
       THEN RAISE EXC_DESPESA_NOEXISTEIX; 
    END IF;
    
    -- El importe del gasto tiene que tener algun valor
    IF (V_IMPORT_DESPESA=0 OR V_IMPORT_DESPESA IS NULL )
        THEN RAISE EXC_IMPORTDES_INVALID;
    END IF;

    --La descripcin no est informada
    IF V_DESCRIPCIO IS NULL
        THEN RAISE EXC_DESCRIPCI_NULL;
    END IF; 
    
    --La justificacin no est informada
    IF V_JUSTIFICACIO IS NULL
        THEN RAISE EXC_JUSTIFICA_NULL;
    END IF; 
    
    --El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
    SELECT COUNT(*) INTO codiestat_existeix
      FROM ESTATDESPESA
      WHERE ESTATDESPESA.CODIESTAT = V_CODIESTAT;
    --DBMS_OUTPUT.put_line('COMPROVACIO ESTAT DE LA DESPESA '||codiestat_existeix);     
    
    IF(codiestat_existeix = 0 )
      THEN RAISE EXC_CODIESTAT_INVALID; 
    END IF;     
    
    --DBMS_OUTPUT.put_line('FORMA PAGAMENT DE LA DESPESA '||V_CODIFORMAPAG);      
    --EL codigo de la forma de pago, tiene que existir en la tabla FORMAPAGAMENT
    SELECT COUNT(*) INTO codiformapag_existeix
      FROM FORMAPAGAMENT
      WHERE FORMAPAGAMENT.CODIFORMAPAG = V_CODIFORMAPAG;
    --DBMS_OUTPUT.put_line('COMPROVACIO FORMA PAGAMENT DE LA DESPESA '||codiformapag_existeix);     
    
    IF(codiformapag_existeix = 0 )
      THEN RAISE EXC_CODIFORMA_INVALID; 
    END IF;  
        
    /* Podemos proceder a modificar el gasto */
    /* Procedemos a guardarnos sus valores porque, despues de borrarlo, tendemos que recalculas las estadicticas */
    
    
    Select DESPESES.DATA_DESPESA, 
           DESPESES.NOMPARLAMENT, 
           DESPESES.CODIPARLAMENTARI, 
           DESPESES.IMPORT_DESPESA

      INTO VM_DATA_DESPESA, 
           VM_NOMPARLAMENT,
           VM_CODIPARLAMENTARI,
           VM_IMPORT_DESPESA
    FROM DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA; 

    /* Actualizamos el gasto de acuerdo con los nuevos valores */
    UPDATE DESPESES SET 
            IMPORT_DESPESA=V_IMPORT_DESPESA,
            TIPUSIVA=V_TIPUSIVA,
            NIFPROVEIDOR=V_NIFPROVEIDOR,
            DESCRIPCIO=V_DESCRIPCIO,
            JUSTIFICACIO=V_JUSTIFICACIO,
            URLDOCUMENTACIO=V_URLDOCUMENTACIO,
            CODIESTAT=V_CODIESTAT,
            codiformapag=v_codiformapag
    WHERE DESPESES.ID_DESPESA=V_ID_DESPESA;    
  
    
    /* Podemos proceder a modificar el gasto */
    /* Procedemos a guardarnos sus valores porque despues de borrarlo, tendemos que recalculas las estadicticas */
    SELECT DESPESES.DATA_DESPESA, 
           DESPESES.NOMPARLAMENT, 
           DESPESES.CODIPARLAMENTARI, 
           DESPESES.IMPORT_DESPESA

      INTO VM_DATA_DESPESA, 
           VM_NOMPARLAMENT,
           VM_CODIPARLAMENTARI,
           VM_IMPORT_DESPESA
    FROM DESPESES WHERE DESPESES.ID_DESPESA=V_ID_DESPESA; 
    
    
    -- INSERTAMOS EN LA TABLA DE lOG, EL RESULTADO DE LA EJECUCION
    RSP:='OK';
    INSERT INTO LOG_PROCEDIMENTS
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP);
    COMMIT;    
      
    /*******************************************/
    /*** aqui: llamar al procedimiento       ***/
    /*** para RECALCULAR LAS ESTADISTICAS !! ***/
    /*******************************************/                    
    
    ESTADISTIQUES (VM_DATA_DESPESA,VM_NOMPARLAMENT,VM_CODIPARLAMENTARI,VM_IMPORT_DESPESA, RSP_estad);
    
    exception 
          when exc_despesa_noexisteix then --El gasto no existe
            RSP:='ERR_MOD_DESP_01: El gasto de cdigo '||V_ID_DESPESA||' No existe';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
            COMMIT;


          WHEN EXC_IMPORTDES_INVALID THEN -- El importe del gasto no esta informado
            RSP:='ERR_MOD_DESP_02: Importe de gasto no informado';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);          
          COMMIT;

          WHEN EXC_DESCRIPCI_NULL THEN --La descripcin no est informada
            RSP:='ERR_MOD_DESP_03: Descripcin del gasto no informada ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);           
          COMMIT;          
 
          WHEN EXC_JUSTIFICA_NULL THEN --La justificacin no est informada
            RSP:='ERR_MOD_DESP_04: Justificacin del gasto no informada ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);            
          commit; 
          
          WHEN EXC_CODIESTAT_INVALID THEN --El codigo del estado en que se encuentra el gasto, tiene que existir en la tabla ESTATDESPESA
            RSP:='ERR_MOD_DESP_05: Codigo de estado de gasto: '||V_CODIESTAT||' invalido o no existente ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);          
          COMMIT;  
          
          WHEN EXC_CODIFORMA_INVALID THEN --EL codigode la forma de pago, tiene que existir en la tabla FORMAPAGAMENT
            RSP:='ERR_MOD_DESP_06: Codigo de la forma de pago: '||V_CODIFORMAPAG||' invalida o no existente ';
            INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          rsp);
          COMMIT;                 
          
          WHEN OTHERS THEN 

            --DBMS_OUTPUT.PUT_LINE(SQLCODE);
            --DBMS_OUTPUT.PUT_LINE(SQLERRM);
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	          COMMIT;
           
END M_DESPESA;

