-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: ESTADISTIQUES 
-- *** REFERENCIA              : ESTADISTIQUES 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** Actualiza las tablas de modulo estadstico cada vez 
-- *** que se produce una modificacin en algn gasto.
-- *** Este procedimiento es DE USO INTERNO; solamente es llamado
-- *** por los procedimientos que modifican los gastos
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** FECHA: Fecha
-- *** PARLAMENTO: Texto
-- *** CODIGO PARLAMENTARIO: Nmero
-- *** IMPORT_DESPESA: Importe del gasto
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ********************************************************************

CREATE OR REPLACE
PROCEDURE  ESTADISTIQUES (
           V_DATA_DESPESA              in DESPESES.DATA_DESPESA%type,
           V_NOMPARLAMENT              in DESPESES.NOMPARLAMENT%type,
           V_CODIPARLAMENTARI          in DESPESES.CODIPARLAMENTARI%type,
           V_IMPORT_DESPESA            IN DESPESES.IMPORT_DESPESA%TYPE,
           RSP                         OUT NOCOPY CLOB) AS

            data_proces VARCHAR2(20);
            nom_proces  VARCHAR2(50);
            param_entrada       CLOB;

            PARLAMENT_EXISTEIX  NUMBER;
            
            any_registre        NUMBER;  -- AO EN EL QUE SE ESTA HACIENDO EL REGISTRO
            
            CURSOR_R            SYS_REFCURSOR; -- Cursor para recoger datos devueltos por las selects           
            
            -- VARIABLES PARA CONSULTA 1 MODULO ESTADISTICO
            ESTAD_1_existent       NUMBER;            
            any_darrer             NUMBER; -- Ao ultimo en el que un parlamento ha tenido un gasto de cualquier tipo
            any_darrer_1           NUMBER; -- (Ao ultimo -1)  en el que un parlamento ha tenido un gasto de cualquier tipo
            any_darrer_2           NUMBER; -- (Ao ultimo -2)  en el que un parlamento ha tenido un gasto de cualquier tipo
            any_darrer_3           NUMBER; -- (Ao ultimo -3)  en el que un parlamento ha tenido un gasto de cualquier tipo           

            Sum_desp_4anys         NUMBER; -- Suma del gasto en los ultimos 4 aos
            
            -- VARIABLES PARA CONSULTA 2 MODULO ESTADISTICO
            ESTAD_2_existent       NUMBER;
            DiferenciaDespesa      NUMBER;
            
            -- VARIABLES PARA CONSULTA 3 MODULO ESTADISTICO
            ESTAD_3_existent       NUMBER;
            ParlamentAmbDesMajors  VARCHAR (40);
            ImportDespeses         NUMBER;

            -- VARIABLES PARA CONSULTA 4 MODULO ESTADISTICO
            ESTAD_4_existent         NUMBER;            
            any_darrer_p             NUMBER;  -- Ao ultimo en el que un parlamentario ha tenido un gasto
            any_darrer_1_p           NUMBER;  -- (Ao ultimo -1)  en el que un parlamentario ha tenido un gasto
            any_darrer_2_p           NUMBER;  -- (Ao ultimo -2)  en el que un parlamentario ha tenido un gasto
            any_darrer_3_p           NUMBER;  -- (Ao ultimo -3)  en el que un parlamentario ha tenido un gasto
            Sum_desp_4anys_p         NUMBER; -- Suma del gasto en los ultimos 4 aos de un parlamentario
            V_nom                    VARCHAR2(40);
            V_cognoms                VARCHAR2(40);

            -- VARIABLES PARA CONSULTA 5 MODULO ESTADISTICO
            ESTAD_5_existent         NUMBER;
            MitjaDelParlamentari     NUMBER;

            -- VARIABLES PARA CONSULTA 6 MODULO ESTADISTICO
            ESTAD_6_existent           NUMBER;
            V_codiparlamentari_max     NUMBER;  
            V_nomparlament_max         VARCHAR2(40);
            V_nom_max                  VARCHAR2(40);
            V_cognom_max               VARCHAR2(40);
            V_despesa_max              NUMBER; 
            V_codiparlamentari_min     NUMBER;  
            V_nomparlament_min         VARCHAR2(40);
            V_nom_min                  VARCHAR2(40);
            V_cognom_min               VARCHAR2(40);
            V_despesa_min              NUMBER; 
            
            -- VARIABLES PARA CONSULTA 7 MODULO ESTADISTICO
            ESTAD_7_EXISTENT           NUMBER;
            V_Nomparlament_Mitja_Major VARCHAR2(40);
            V_Mitja_Major              NUMBER;
       
            -- VARIABLES PARA CONSULTA 8 MODULO ESTADISTICO
            ESTAD_8_EXISTENT           NUMBER;
            V_DespesaTotalAny          NUMBER;
            
Begin
        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='ESTADISTIQUES';
        param_entrada := (
           'DATA_DESPESA:'||V_DATA_DESPESA||'---'||
           'NOMPARLAMENT:'||V_NOMPARLAMENT||'---'||
           'CODIPARLAMENTARI:'||V_CODIPARLAMENTARI||'---'||              
           'IMPORT_DESPESA:'||V_IMPORT_DESPESA||'---'  
          );
         
       -- ALGUNOS CALCULOS PREVIOS QUE NOS SERVIRAN PARA VARIOS DE LOS APARTADOS DEL MODULO ESTADISTICO
       -- Ao del registro para el que estamos haciendo un gasto  
       
       any_registre:= EXTRACT(YEAR FROM V_DATA_DESPESA)  ;
       /*****************************************/  
       /******* PUNTO 1 MODULO ESTADISTICO ******/
       /*****************************************/
       
       
       -- Obtenemos el ultimo ao para el que hay gastos para ese parlamento

       SELECT MAX ( EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)) INTO any_darrer FROM DESPESES WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT;
       
       -- Con ello, calculamos cuales son los tres aos anteriores
       
       any_darrer_1 := any_darrer - 1;
       any_darrer_2 := any_darrer - 2;
       any_darrer_3 := any_darrer - 3;

       --DBMS_OUTPUT.put_line(any_darrer||'  '||any_darrer_1||'  '||any_darrer_2||'  '||any_darrer_3);

       -- Calculamos la suma de los gastos de ese parlamento para los 4 ultimos aos
       SELECT COUNT(*) INTO PARLAMENT_EXISTEIX
            FROM DESPESES
       WHERE DESPESES.NOMPARLAMENT = V_NOMPARLAMENT;
       
       IF (PARLAMENT_EXISTEIX>0) THEN
           SELECT SUM (DESPESES.IMPORT_DESPESA) into Sum_desp_4anys
           FROM DESPESES
           WHERE 
              DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND
              (EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer OR
              EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_1 OR
              EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_2 OR
              EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_3)
           GROUP BY DESPESES.NOMPARLAMENT;
       ELSE Sum_desp_4anys:=0;
       END IF;
       
       --DBMS_OUTPUT.put_line(Sum_desp_4anys);
       
       -- Miramos si el parlamento ya existe en la tabla de estadisticas correspondiente
       -- Si no existe, insertamos el registro correspondiente
       -- Si existen solamente actualizamos el registro.
       
       SELECT COUNT(*) INTO ESTAD_1_existent
          FROM ESTAD_1
       WHERE ESTAD_1.NOMPARLAMENT = V_NOMPARLAMENT;
          
       --DBMS_OUTPUT.put_line(ESTAD_1_existent);
       
       IF (estad_1_existent=0) THEN
          INSERT INTO ESTAD_1 (NOMPARLAMENT,IMPORT_DESPESA,PERIODE) 
       VALUES (V_NOMPARLAMENT,Sum_desp_4anys, any_darrer_3||'-'||any_darrer);
       else
          UPDATE ESTAD_1 SET 
            IMPORT_DESPESA=Sum_desp_4anys,
            PERIODE=any_darrer_3||'-'||any_darrer
          where ESTAD_1.NOMPARLAMENT=V_NOMPARLAMENT;
       end if;

       /*****************************************/  
       /******* PUNTO 2 MODULO ESTADISTICO ******/
       /*****************************************/

       -- Calculamos la diferencia entre el parlamentario que gasta mas y el que gasta menos
       -- Para el parlamento y ao al que corresponde el registro del gasto
       
       -- Solamente actualizamos la tabla de estadicticas
       -- si se trata de un gasto de parlamentario
       if V_CODIPARLAMENTARI IS NOT NULL THEN
           SELECT 
             MAX (SUM (DESPESES.IMPORT_DESPESA))-MIN (SUM (DESPESES.IMPORT_DESPESA)) AS DIFERENCIA INTO DiferenciaDespesa
           FROM DESPESES
             WHERE 
               DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND
               DESPESES.TIPUSDESPESA='PARLAMENTARI'
             AND EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)=any_registre
           GROUP BY DESPESES.CODIPARLAMENTARI;
           
    
           -- Miramos si el parlamento ya existe en la tabla de estadisticas correspondiente
           -- Si no existe, insertamos el registro correspondiente
           -- Si existen solamente actualizamos el registro.
    
           SELECT COUNT(*) INTO ESTAD_2_existent
              FROM ESTAD_2
           WHERE 
              ESTAD_2.NOMPARLAMENT = V_NOMPARLAMENT AND
              ESTAD_2.ANY_ESTAD = any_registre;
              
           --DBMS_OUTPUT.put_line(ESTAD_2_existent);
           
           IF (ESTAD_2_existent=0) THEN
              INSERT INTO ESTAD_2 (NOMPARLAMENT,ANY_ESTAD,IMPORT_DIFERENCIA) 
           VALUES (V_NOMPARLAMENT,any_registre, DiferenciaDespesa);
           else
              UPDATE ESTAD_2 SET 
                IMPORT_DIFERENCIA=DiferenciaDespesa
              WHERE
                ESTAD_2.NOMPARLAMENT=V_NOMPARLAMENT AND
                ESTAD_2.ANY_ESTAD=any_registre;
           END IF;
       END IF;-- del if V_CODIPARLAMENTARI<> NULL    

       /*****************************************/  
       /******* PUNTO 3 MODULO ESTADISTICO ******/
       /*****************************************/

      -- Obtebemos la lista de gastos por parlamento
      -- para el ao del gasto que estamos considerando
      -- Ordenandolos por gastos (de mayor a menos)
      -- De este modo obtendremos en la primera,fila, el parlamento con un gasto maximo
      
      OPEN CURSOR_R FOR
        SELECT 
                DESPESES.NOMPARLAMENT AS NOMPARLAMENT, SUM(DESPESES.IMPORT_DESPESA) AS TOTAL_DESPESES
        FROM DESPESES
        WHERE
        EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)=any_registre
        GROUP BY DESPESES.NOMPARLAMENT, EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)
        ORDER BY  SUM(DESPESES.IMPORT_DESPESA) DESC;
        
       --dbms_output.put_line('HOLA');  
      
      -- Recupero la primera fila de la select
      -- es decir el parlamento con maximo numero de gastos ese ao
      -- (esta en la primera de las filas recuperadas)
      FETCH CURSOR_R INTO ParlamentAmbDesMajors, ImportDespeses;
      CLOSE CURSOR_R;

      -- Miramos si el ao ya existe en la tabla de estadisticas correspondiente
      -- Si no existe, insertamos el registro correspondiente
      -- Si existen solamente actualizamos el registro.
      
      SELECT COUNT(*) INTO ESTAD_3_existent
         FROM ESTAD_3
      WHERE 
         ESTAD_3.ANY_ESTAD = any_registre;
         
      --DBMS_OUTPUT.put_line(ESTAD_3_existent);

      IF (ESTAD_3_existent=0) THEN
         INSERT INTO ESTAD_3 (ANY_ESTAD,NOMPARLAMENT,MAXIM_DESPESA) 
      VALUES (any_registre,ParlamentAmbDesMajors,ImportDespeses);
      else
         UPDATE ESTAD_3 SET 
           ESTAD_3.NOMPARLAMENT=ParlamentAmbDesMajors,
           ESTAD_3.MAXIM_DESPESA=ImportDespeses
         WHERE
           ESTAD_3.ANY_ESTAD=any_registre;
      END IF;

       /*****************************************/  
       /******* PUNTO 4 MODULO ESTADISTICO ******/
       /*****************************************/
            
       if V_CODIPARLAMENTARI IS NOT NULL THEN     
           -- Obtenemos el ultimo ao para el que hay gastos para ese parlamento            
           SELECT MAX ( EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)) INTO any_darrer_p 
           FROM DESPESES 
           WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT and DESPESES.CODIPARLAMENTARI=V_CODIPARLAMENTARI;
           
           -- Con ello, calculamos cuales son los tres aos anteriores
           
           any_darrer_1_p := any_darrer_p - 1;
           any_darrer_2_p := any_darrer_p - 2;
           any_darrer_3_p := any_darrer_p - 3;
    
           --DBMS_OUTPUT.put_line(any_darrer_p||'  '||any_darrer_1_p||'  '||any_darrer_2_p||'  '||any_darrer_3_p);
           
           OPEN CURSOR_R FOR
             select 
                SUM (DESPESES.IMPORT_DESPESA),
                PARLAMENTARIS.NOM,
                PARLAMENTARIS.COGNOMS
                FROM DESPESES
                
                INNER JOIN PARLAMENTARIS ON
                DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT AND
                DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI
                
                WHERE ( 
                EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_p OR
                EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_1_p OR
                EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_2_p OR
                EXTRACT(YEAR FROM DESPESES.DATA_DESPESA) = any_darrer_3_p)
                AND DESPESES.tipusdespesa='PARLAMENTARI'
                AND DESPESES.NOMPARLAMENT=V_NOMPARLAMENT
                AND DESPESES.CODIPARLAMENTARI=V_CODIPARLAMENTARI
             GROUP BY DESPESES.NOMPARLAMENT,DESPESES.CODIPARLAMENTARI,PARLAMENTARIS.NOM,PARLAMENTARIS.COGNOMS;
        
           FETCH CURSOR_R INTO Sum_desp_4anys_p, V_nom, V_cognoms;
           CLOSE CURSOR_R;
  
  
         -- Miramos si el parlamentario
         -- Si no existe, insertamos el registro correspondiente
         -- Si existen solamente actualizamos el registro.
  
           SELECT COUNT(*) INTO ESTAD_4_existent
              FROM ESTAD_4
           WHERE 
              ESTAD_4.NOMPARLAMENT = V_NOMPARLAMENT AND
              ESTAD_4.CODIPARLAMENTARI = V_CODIPARLAMENTARI;
              
           --DBMS_OUTPUT.put_line(ESTAD_4_existent);
    
           IF (ESTAD_4_existent=0) THEN
              INSERT INTO ESTAD_4 (CODIPARLAMENTARI,NOMPARLAMENT,NOM,COGNOMS,PERIODE,DESPESA_PERIODE) 
           VALUES (V_CODIPARLAMENTARI,V_NOMPARLAMENT,V_nom,V_cognoms,any_darrer_3_p||'-'||any_darrer_p,Sum_desp_4anys_p);
           else
              UPDATE ESTAD_4 SET 
                ESTAD_4.NOM=V_nom,
                ESTAD_4.COGNOMS=V_cognoms,
                ESTAD_4.PERIODE=any_darrer_3_p||'-'||any_darrer_p,                
                ESTAD_4.DESPESA_PERIODE=Sum_desp_4anys_p
              WHERE
              ESTAD_4.NOMPARLAMENT = V_NOMPARLAMENT AND
              ESTAD_4.CODIPARLAMENTARI = V_CODIPARLAMENTARI;
           END IF;
       end if; -- del if V_CODIPARLAMENTARI<> NULL    

       /*****************************************/  
       /******* PUNTO 5 MODULO ESTADISTICO ******/
       /*****************************************/
     
      IF V_CODIPARLAMENTARI IS NOT NULL THEN          
          SELECT   -- Con esta select obtengo la media anual del parlamentario
                   -- La calculo como la suma de los gastos directos de todos los parlamentarios de todos los parlamentos
                   -- Dividida por el numero total de parlamentarios que han tenido algun gasto directo ese ao
                   -- Dicho valor, lo obtengo directamente en la variable :***** MitjaDelParlamentari ******
             Round
             (
                 (    -- SUMA DE GASTOS DE TODOS LOS PARLAMENMTARIOS DE TODOS LOS PARLAMENTOS PARA EL AO CONSIDERADO
                    SELECT SUM (DESPESES.IMPORT_DESPESA) AS TOTAL_PARLAMENTARIS_ANY  
                    from DESPESES 
                    WHERE DESPESES.TIPUSDESPESA='PARLAMENTARI'  AND 
                    ( EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=any_registre)
                  )/( -- NUMERO DE PARLAMENTARIOS QUE TIENEN ALGUN GASTO REGISTRADO PARA EL AO CONSIDERADO
                    SELECT COUNT (*) AS TOTAL_PARLAMENT_AMB_DESPESA FROM (
                    Select DISTINCT DESPESES.CODIPARLAMENTARI, DESPESES.NOMPARLAMENT 
                    from DESPESES
                    WHERE DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
                    EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=any_registre )   
                )
               ,2) into MitjaDelParlamentari 
          FROM DESPESES 
          WHERE EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=any_registre
          GROUP BY EXTRACT(YEAR FROM DESPESES.DATA_DESPESA);
          
          
         -- Miramos si existe el registro correspondiente al ao que estamos considerando
         -- En la tabla ESTAD_5
         -- Si no existe, insertamos el registro correspondiente
         -- Si existen solamente actualizamos el registro.
  
          SELECT COUNT(*) INTO ESTAD_5_existent
             FROM ESTAD_5
          WHERE 
             ESTAD_5.ANY_ESTAD = any_registre;
             
          --DBMS_OUTPUT.put_line(ESTAD_5_existent);
          
          IF (ESTAD_5_existent=0) THEN
             INSERT INTO ESTAD_5 (ANY_ESTAD,DESPESA_MITJA) 
          VALUES (any_registre,MitjaDelParlamentari);
          else
             UPDATE ESTAD_5 SET 
               ESTAD_5.DESPESA_MITJA=MitjaDelParlamentari
             WHERE
               ESTAD_5.ANY_ESTAD = any_registre;
          END IF;
          
      end if; -- del if V_CODIPARLAMENTARI<> NULL    
      
       /*****************************************/  
       /******* PUNTO 6 MODULO ESTADISTICO ******/
       /*****************************************/    
 
            
      IF V_CODIPARLAMENTARI IS NOT NULL THEN       
          
          -- Obtenemos el parlamentario que ha otenido mas gastos
          -- La select utilizada , devuelve los registros por importe descendente
          -- De este modo nos aseguramos de que el primero es el del parlamentario con mayor gasto
          
          OPEN CURSOR_R FOR
            SELECT desp, codpar, nompar, PARLAMENTARIS.NOM, PARLAMENTARIS.COGNOMS
            FROM
            (
                SELECT
                 SUM (DESPESES.IMPORT_DESPESA) AS desp, 
                 DESPESES.CODIPARLAMENTARI AS codpar, 
                 DESPESES.NOMPARLAMENT AS nompar
                FROM DESPESES
                WHERE DESPESES.TIPUSDESPESA='PARLAMENTARI' AND eXTRACT (YEAR FROM DATA_DESPESA)=any_registre
                GROUP BY DESPESES.CODIPARLAMENTARI, DESPESES.NOMPARLAMENT, EXTRACT (YEAR FROM DATA_DESPESA)
                ORDER BY  SUM (DESPESES.IMPORT_DESPESA) DESC
            )
            Inner join PARLAMENTARIS ON
               CODPAR=PARLAMENTARIS.CODIPARLAMENTARI AND
               nompar=PARLAMENTARIS.NOMPARLAMENT;
          
          -- Recupero la primera fila de la select
          -- es decir el parlamentario con maximo gasto ese ao
          -- (esta en la primera de las filas recuperadas)
          Fetch CURSOR_R INTO 
            V_despesa_max,
            V_codiparlamentari_max,
            V_nomparlament_max,
            V_nom_max,
            V_cognom_max;
          CLOSE CURSOR_R;
         

          -- Luego obtenemos el parlamentario que ha otenido MENOS gastos
          -- Lo hacemos con la misma select que en el caso anterior
          -- pero ordennado los registros en orden ascendente
          -- De este modo nos aseguramos de que elprimero corresponde al parlamentario de menos gasto
          
          OPEN CURSOR_R FOR
            SELECT desp, codpar, nompar, PARLAMENTARIS.NOM, PARLAMENTARIS.COGNOMS
            FROM
            (
                SELECT
                 SUM (DESPESES.IMPORT_DESPESA) AS desp, 
                 DESPESES.CODIPARLAMENTARI AS codpar, 
                 DESPESES.NOMPARLAMENT AS nompar
                FROM DESPESES
                WHERE DESPESES.TIPUSDESPESA='PARLAMENTARI' AND EXTRACT (YEAR FROM DATA_DESPESA)=any_registre
                GROUP BY DESPESES.CODIPARLAMENTARI, DESPESES.NOMPARLAMENT, EXTRACT (YEAR FROM DATA_DESPESA)
                ORDER BY  SUM (DESPESES.IMPORT_DESPESA) ASC
            )
            Inner join PARLAMENTARIS ON
               codpar=PARLAMENTARIS.CODIPARLAMENTARI AND
               NOMPAR=PARLAMENTARIS.NOMPARLAMENT;
               
          -- Recupero la primera fila de la select
          -- es decir el parlamentario con minimo gasto ese ao
          -- (esta en la primera de las filas recuperadas)               
          Fetch CURSOR_R INTO 
            V_despesa_min,
            V_codiparlamentari_min,
            V_nomparlament_min,
            V_nom_min,
            V_cognom_min;
          CLOSE CURSOR_R;


         -- Miramos si existe el registro correspndiente al ao que estamos considerando
         -- Si no existe, insertamos el registro correspondiente
         -- Si existe, solamente actualizamos el registro.
          SELECT COUNT(*) INTO ESTAD_6_existent
             FROM ESTAD_6
          WHERE 
             ESTAD_6.ANY_ESTAD = any_registre;
             
          --DBMS_OUTPUT.put_line(ESTAD_6_existent);
          
          IF (ESTAD_6_existent=0) THEN
             INSERT INTO ESTAD_6 (
                   ANY_ESTAD,
                   CODIPARLAMENTARI_MAX,
                   NOMPARLAMENT_MAX,
                   NOM_MAX,
                   COGNOMS_MAX,
                   DESPESA_MAX,
                   CODIPARLAMENTARI_MIN,
                   NOMPARLAMENT_MIN,
                   NOM_MIN,
                   COGNOMS_MIN,
                   DESPESA_MIN
                   ) 
             VALUES (
                   any_registre,
                   V_codiparlamentari_max,     
                   V_nomparlament_max,  
                   V_nom_max,  
                   V_cognom_max,  
                   V_despesa_max,      
                   V_codiparlamentari_min,    
                   V_nomparlament_min,     
                   V_nom_min,              
                   V_cognom_min,             
                   V_despesa_min                
                 );
          else
             UPDATE ESTAD_6 SET 
                   CODIPARLAMENTARI_MAX=V_codiparlamentari_max,
                   NOMPARLAMENT_MAX=V_nomparlament_max,
                   NOM_MAX=V_nom_max,
                   COGNOMS_MAX=V_cognom_max,
                   DESPESA_MAX=V_despesa_max,
                   CODIPARLAMENTARI_MIN=V_codiparlamentari_min,
                   NOMPARLAMENT_MIN=V_nomparlament_min,
                   NOM_MIN=V_nom_min,
                   COGNOMS_MIN=V_cognom_min,
                   DESPESA_MIN=V_despesa_min
             WHERE
                  ESTAD_6.ANY_ESTAD=any_registre;
          END IF; 
      end if; -- del if V_CODIPARLAMENTARI<> NULL   
         
       /*****************************************/  
       /******* PUNTO 7 MODULO ESTADISTICO ******/
       /*****************************************/    
   
      IF V_CODIPARLAMENTARI IS NOT NULL THEN   
          -- Calculamos la media del gasto por parlamentario,
          -- Para cada uno de los parlamentos, para el ao considerado
          -- Y la ordenamos de mayor a menor gasto medio
          OPEN CURSOR_R FOR
              SELECT
                ROUND (
                    -- Gastos de los parlamentarios para el parlamento yao considerados 
                     SUM (DESPESES1.IMPORT_DESPESA
                      )/( 
                    -- Numero de parlamentarios para ese parlamento y ao considerado
                    select count (distinct despeses2.CODIPARLAMENTARI ) from despeses despeses2
                    where 
                    extract (year from despeses1.data_despesa) = extract (year from despeses2.data_despesa)
                    and despeses1.NOMPARLAMENT=despeses2.NOMPARLAMENT
                    AND despeses2.tipusdespesa='PARLAMENTARI'
                ),2) AS MEDIA,
                despeses1.NOMPARLAMENT
              FROM DESPESES DESPESES1  
              WHERE despeses1.tipusdespesa='PARLAMENTARI' 
              AND ( EXTRACT (YEAR FROM despeses1.data_despesa)=any_registre ) 
              GROUP BY EXTRACT (YEAR FROM despeses1.data_despesa), despeses1.NOMPARLAMENT
              ORDER BY MEDIA DESC;

          --Nos quedamos con el primer registro  
          Fetch CURSOR_R INTO V_Mitja_Major, V_Nomparlament_Mitja_Major;
          CLOSE CURSOR_R;
          
          SELECT COUNT(*) INTO ESTAD_7_existent
             FROM ESTAD_7
          WHERE 
             ESTAD_7.ANY_ESTAD = any_registre;
             
          IF ESTAD_7_EXISTENT=0 THEN
             INSERT INTO ESTAD_7(NOMPARLAMENT,ANY_ESTAD,MITJA_DESPESA) 
             VALUES (V_Nomparlament_Mitja_Major,any_registre,V_Mitja_Major);
          ELSE
             UPDATE Estad_7 SET
               NOMPARLAMENT=V_Nomparlament_Mitja_Major,
               MITJA_DESPESA=V_Mitja_Major
             WHERE ANY_ESTAD=any_registre;
          END IF;
      end if; -- del if V_CODIPARLAMENTARI<> NULL  


       /*****************************************/  
       /******* PUNTO 8 MODULO ESTADISTICO ******/
       /*****************************************/           

        SELECT SUM (DESPESES.IMPORT_DESPESA) INTO V_DespesaTotalAny
        FROM DESPESES
        WHERE EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)=any_registre;

        SELECT COUNT(*) INTO ESTAD_8_existent
           FROM ESTAD_8
        WHERE 
           ESTAD_8.ANY_ESTAD = any_registre;

        IF ESTAD_8_existent=0 THEN
          INSERT INTO ESTAD_8 (ANY_ESTAD, DESPESA) VALUES (any_registre,V_DespesaTotalAny);
        ELSE
          UPDATE ESTAD_8 SET DESPESA=V_DespesaTotalAny WHERE ANY_ESTAD=any_registre;
        END IF;
          
    ----------------
    -- EXCEPCIONES--
    ----------------
    
    EXCEPTION
        
        WHEN OTHERS THEN 
          DBMS_OUTPUT.PUT_LINE(SQLCODE);
          DBMS_OUTPUT.PUT_LINE(SQLERRM);
          RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
          INSERT INTO LOG_PROCEDIMENTS
                VALUES (SEQ_ID_LOG.NextVal,
                        data_proces,
                        nom_proces,
                        param_entrada,
                        NULL,
                        RSP);
        commit;        
        
END ESTADISTIQUES;



    