-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_DespesesAnyParlamentari 
-- *** REFERENCIA              : A 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Elabora un listado de gastos asociados un parlamentario 
-- *** durante un ao determinado, por orden descendiente del gasto
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** AO: Numero
-- *** CODIGO DE PARLAMENTARIO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : "OK" si el resultado es correcto
-- ***       Codigo y descrip. del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_A_01: El parlamento no existe
-- *** ERROR_PROC_A_02: El parlamentario no existe en ese parlamento
-- *** ERROR_PROC_A_03: No hay gastos para ese parlamentario en el ao solicitado
-- ******************************************************************** 

CREATE OR REPLACE
PROCEDURE proc_DespesesAnyParlamentari(V_PARLAMENT IN VARCHAR2, V_ANY IN NUMBER, V_CODIPARLAMENTARI IN  NUMBER, RSP OUT NOCOPY CLOB) AS

    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;
        
    S_ID_DESPESA         DESPESES.ID_DESPESA%type;
    S_NOM                PARLAMENTARIS.NOM%type;
    S_COGNOMS            PARLAMENTARIS.COGNOMS%type;
    S_DATA_DESPESA       DESPESES.DATA_DESPESA%type;
    S_CODICATDESPESA     DESPESES.CODICATDESPESA%type;
    S_IMPORT_DESPESA     DESPESES.IMPORT_DESPESA%type;
    S_DESCRIPCIO         DESPESES.DESCRIPCIO%type;
    S_JUSTIFICACIO       DESPESES.JUSTIFICACIO%type;
    S_NOMPARLAMENT       DESPESES.NOMPARLAMENT%type;
    S_CODIPARLAMENTARI   DESPESES.CODIPARLAMENTARI%type;


    CURSOR_R             SYS_REFCURSOR;


    EXC_NO_PARLAMENT                 EXCEPTION;
    ERROR_PARLAMENT                  NUMBER;

    EXC_NO_PARLAMENT_PARLAMENTARI    EXCEPTION;
    ERROR_PARLAMENTARI               NUMBER;
    
    EXC_NO_DESPESA_PARLAMENTARI      EXCEPTION;
    ERROR_DESPESA                    NUMBER;
    

    BEGIN
                     
        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='proc_DespesesAnyParlamentari';
        param_entrada := (
          'PARLAMENT:'||V_PARLAMENT||'---'||
          'ANY:' ||V_ANY||'---'||
          'CODIPARLAMENTARI:'||V_CODIPARLAMENTARI     
          );
         
        -- EL PARLAMENTO NO EXISTE  
        SELECT COUNT (*) INTO ERROR_PARLAMENT FROM PARLAMENTS
        WHERE PARLAMENTS.NOMPARLAMENT =V_PARLAMENT;
        IF ERROR_PARLAMENT=0  THEN RAISE EXC_NO_PARLAMENT; END IF;
        
        -- EL PARLAMENTARIO NO EXISTE PARA ESE PARLAMENTO
        SELECT COUNT(*) into ERROR_PARLAMENTARI from
        PARLAMENTARIS WHERE
        PARLAMENTARIS.CODIPARLAMENTARI=V_CODIPARLAMENTARI AND
        PARLAMENTARIS.NOMPARLAMENT=V_PARLAMENT;

        IF ERROR_PARLAMENTARI=0  THEN RAISE EXC_NO_PARLAMENT_PARLAMENTARI; END IF;
        
        -- NO HAN OCURRIDO GASTOS EN EL AO SELECCIONADO   
        SELECT COUNT(*) into ERROR_DESPESA from
        DESPESES WHERE
        DESPESES.CODIPARLAMENTARI=V_CODIPARLAMENTARI AND
        DESPESES.NOMPARLAMENT=V_PARLAMENT AND
        EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY;
        
        IF ERROR_DESPESA=0  THEN RAISE EXC_NO_DESPESA_PARLAMENTARI; END IF;   
        
        OPEN CURSOR_R FOR
        Select 
             DESPESES.ID_DESPESA,
             PARLAMENTARIS.NOM,
             PARLAMENTARIS.COGNOMS,
             DESPESES.DATA_DESPESA,
             DESPESES.CODICATDESPESA,
             DESPESES.IMPORT_DESPESA,
             DESPESES.DESCRIPCIO,
             DESPESES.JUSTIFICACIO,
             DESPESES.NOMPARLAMENT,
             DESPESES.CODIPARLAMENTARI
         from DESPESES 
         inner join  PARLAMENTARIS ON ( DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND  DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT)
         where 
            EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY
         AND DESPESES.nomparlament=V_PARLAMENT
         AND PARLAMENTARIS.codiparlamentari=V_CODIPARLAMENTARI 
         order by DESPESES.IMPORT_DESPESA DESC;

  
         DBMS_OUTPUT.PUT_LINE ('***** LISTA LOS GASTOS DIRECTOS DEL PARLAMENTARIO: '||V_CODIPARLAMENTARI );
         DBMS_OUTPUT.PUT_LINE ('***** PERTENECIENTES AL PARLAMENTO: '|| V_PARLAMENT);
         DBMS_OUTPUT.PUT_LINE ('***** DURANTE EL AO: '|| V_ANY);
         
         DBMS_OUTPUT.PUT_LINE    (LPAD ('ID GASTO',10) ||
                                  LPAD ('NOMBRE',15)||
                                  LPAD ('APELLIDOS',20)||
                                  LPAD ('FECHA',10)||
                                  LPAD ('IMPORTE',10)||
                                  LPAD ('DESCRIPCION',25) ||
                                  LPAD ('JUSTIFICACION',30)||
                                  LPAD ('PARLAMENTO',25)||
                                  LPAD ('PARLAMENTARIO',15));

         LOOP FETCH CURSOR_R INTO S_ID_DESPESA,
                                 S_NOM,
                                 S_COGNOMS,
                                 S_DATA_DESPESA,
                                 S_CODICATDESPESA,
                                 S_IMPORT_DESPESA,
                                 S_DESCRIPCIO,
                                 S_JUSTIFICACIO,
                                 S_NOMPARLAMENT,
                                 S_CODIPARLAMENTARI;
        EXIT WHEN CURSOR_R%NOTFOUND;  
        DBMS_OUTPUT.PUT_LINE    (LPAD (S_ID_DESPESA, 10)||
                                 LPAD (S_NOM, 15)||
                                 LPAD (S_COGNOMS, 20)||
                                 LPAD (S_DATA_DESPESA, 10)||
                                 LPAD (S_IMPORT_DESPESA, 10)||
                                 LPAD (S_DESCRIPCIO, 25)||
                                 LPAD (S_JUSTIFICACIO, 30)||
                                 LPAD (S_NOMPARLAMENT, 25)||
                                 LPAD (S_CODIPARLAMENTARI, 15));

        END LOOP;
	        
        RSP:='OK';
        param_sortida := 'proc_DespesesAnyParlamentari ejecutado correctamente';
        INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                param_sortida,
                RSP);
        COMMIT;
        
        ----------------
        -- EXCEPCIONES--
        ----------------
        
        
        EXCEPTION
        WHEN EXC_NO_PARLAMENT THEN
        RSP:='ERROR_PROC_A_01: El parlamento:  '||V_CODIPARLAMENTARI||' no existe ';
        INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                NULL,
                RSP); 
        COMMIT;        
        
        
        WHEN EXC_NO_PARLAMENT_PARLAMENTARI THEN
        RSP:='ERROR_PROC_A_02: El parlamentario de codigo:  '||V_CODIPARLAMENTARI||' no existe para el parlamento:'|| V_PARLAMENT;
        INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                NULL,
                RSP); 
        COMMIT;

        WHEN EXC_NO_DESPESA_PARLAMENTARI THEN
        RSP:='ERROR_PROC_A_03: El parlamentario de codigo:  '||V_CODIPARLAMENTARI||' no tiene gastos en el ao :'|| V_ANY; 
        INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                NULL,
                RSP); 
        COMMIT;

        WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	       COMMIT;
        
END proc_DespesesAnyParlamentari;


/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  BEGIN
      PROC_DESPESESANYPARLAMENTARI('PARLAMENT DE CATALUNYA', 2009,1, RSP ) ;
      ULTIMA_LINEA_LOG;
end;
*/
