-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_ParlamentarisDespDir 
-- *** REFERENCIA              : B 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** 
-- *** Elabora un listado de parlamentarios con su gasto directo,
-- *** incluyendo nombre, apellidos, partido poltico en el momento
-- *** de registrar el gasto, suma de los gastos directos de cada parlamentario
-- *** 
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** AO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : Devuelve "OK" si el resultado es correcto
-- ***       Codigo y descripcion del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_B_01: El parlamento no existe
-- *** ERROR_PROC_B_02: No hay gastos para ese parlamento en el ao solicitado
-- *** 
-- ******************************************************************** 

CREATE OR REPLACE
PROCEDURE proc_ParlamentarisDespDir (V_PARLAMENT IN VARCHAR2, V_ANY IN NUMBER, RSP OUT NOCOPY CLOB) AS


    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;       
    
    S_CODIPARLAMENTARI   DESPESES.CODIPARLAMENTARI%type;
    S_NOM                PARLAMENTARIS.NOM%type;
    S_COGNOMS            PARLAMENTARIS.COGNOMS%type;
    S_PARTITPOL          PARLAMENTARIS.PARTITPOL%type;        
    S_SUMA_DESPESES      NUMBER; 

    CURSOR_R             SYS_REFCURSOR;


    -- NO EXISTE EL PARLAMENTO
    EXC_NO_PARLAMENT    EXCEPTION;
    ERROR_PARLAMENT     NUMBER;
    
    -- NO EXISTEN GASTOS PARA ESE PARLAMENTO Y ESE AO
    EXC_NO_DESPESA_PARLAMENT      EXCEPTION;
    ERROR_DESPESA                  NUMBER;

    BEGIN
                     
        data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nom_proces:='proc_ParlamentarisDespDir';
        param_entrada := (
          'PARLAMENT:'||V_PARLAMENT||'---'||
          'ANY:' ||V_ANY||'---'
          );
          
        
        SELECT COUNT(*) INTO ERROR_PARLAMENT FROM
        PARLAMENTS WHERE
        PARLAMENTS.NOMPARLAMENT=V_PARLAMENT;

        IF ERROR_PARLAMENT=0  THEN RAISE EXC_NO_PARLAMENT; END IF;


        SELECT COUNT(*) INTO ERROR_DESPESA 
        FROM PARLAMENTARIS   
           INNER JOIN DESPESES ON DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT
        WHERE  EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY AND DESPESES.NOMPARLAMENT=V_PARLAMENT; 
        
        IF ERROR_DESPESA=0  THEN RAISE EXC_NO_DESPESA_PARLAMENT; END IF;   

  
  
        OPEN CURSOR_R FOR  
        select
           DESPESES.CODIPARLAMENTARI,
           PARLAMENTARIS.NOM,
           PARLAMENTARIS.COGNOMS,
           PARLAMENTARIS.PARTITPOL,
           --EXTRACT(YEAR FROM DESPESES.DATA_DESPESA),
           SUM (DESPESES.IMPORT_DESPESA)
        from PARLAMENTARIS   
           inner join DESPESES ON DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT
        where  EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY and
        DESPESES.NOMPARLAMENT=V_PARLAMENT 
        group by DESPESES.CODIPARLAMENTARI, PARLAMENTARIS.NOM, PARLAMENTARIS.COGNOMS, PARLAMENTARIS.PARTITPOL,  extract(year from DESPESES.DATA_DESPESA)
        ORDER BY SUM (DESPESES.IMPORT_DESPESA) DESC;
        
        
        dbms_output.put_line ('***** SUMA DE LOS GASTOS DIRECTOS DE LOS PARLAMENTARIOS');
        dbms_output.put_line ('***** PERTENECIENTES AL PARLAMENTO: '|| V_PARLAMENT);
        DBMS_OUTPUT.PUT_LINE ('***** DURANTE EL AO: '|| V_ANY);
        
        DBMS_OUTPUT.PUT_LINE    (LPAD ('PARLAMENTARIO', 10)||
                                 LPAD ('NOMBRE', 15)||
                                 LPAD ('APELLIDOS', 20)||
                                 LPAD ('PARTIDO', 25)||
                                 LPAD ('GASTO', 10)); 
                                 
        LOOP FETCH CURSOR_R INTO S_CODIPARLAMENTARI,
                                 S_NOM,
                                 S_COGNOMS,
                                 S_PARTITPOL,
                                 S_SUMA_DESPESES;
        EXIT WHEN CURSOR_R%NOTFOUND;  
        DBMS_OUTPUT.PUT_LINE    (LPAD (S_CODIPARLAMENTARI, 10)||
                                 LPAD (S_NOM, 15)||
                                 LPAD (S_COGNOMS, 20)||
                                 LPAD (S_PARTITPOL, 25)||
                                 LPAD (S_SUMA_DESPESES,10));

        END LOOP;
	        
        RSP:='OK';
        param_sortida := 'proc_ParlamentarisDespDir ejecutado correctamente';
        INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                param_sortida,
                RSP);
        COMMIT;
        
        ----------------
        -- EXCEPCIONES--
        ----------------
    
        EXCEPTION
          WHEN EXC_NO_PARLAMENT THEN
          RSP:=('ERROR_PROC_B_01: El parlamento:  '||V_PARLAMENT||' no existe');
          INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP); 
          COMMIT;

        WHEN EXC_NO_DESPESA_PARLAMENT THEN
          RSP:=('ERROR_PROC_B_02: El parlamento :  '||V_PARLAMENT||' no tiene gastos en el ao :'|| V_ANY); 
          INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
          VALUES (SEQ_ID_LOG.NextVal,
                  data_proces,
                  nom_proces,
                  param_entrada,
                  NULL,
                  RSP); 
          COMMIT;


        WHEN OTHERS THEN 

         --DBMS_OUTPUT.PUT_LINE(SQLCODE);
         --DBMS_OUTPUT.PUT_LINE(SQLERRM);
         
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO LOG_PROCEDIMENTS
                  VALUES (SEQ_ID_LOG.NextVal,
                          data_proces,
                          nom_proces,
                          param_entrada,
                          NULL,
                          RSP);
	       COMMIT;    
END proc_ParlamentarisDespDir;


/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  BEGIN
      proc_ParlamentarisDespDir ('DEUSTCHER BUNDESTAG',2011, RSP );
      ULTIMA_LINEA_LOG;
end;
*/
