-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_DespTotalTotsParlam 
-- *** REFERENCIA              : C 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** Elabora un listado que incluye para un ao determinado:
-- *** Nombre del parlamento, n de parlamentarios, gasto medio
-- *** de los parlamentarios de ese parlamento, gasto imputable a
-- *** los parlamentarios, gasto imputable solo al parlamento,
-- *** gasto total del parlamento
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** AO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : Devuelve "OK" si el resultado es correcto
-- ***       Codigo y descripcion del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_C_01: No se registraron gastos en el ao solicitado
-- ******************************************************************** 

CREATE OR REPLACE
PROCEDURE proc_DespTotalTotsParlam (V_ANY IN NUMBER, RSP OUT NOCOPY CLOB) AS


    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;

    -- VARIBALES PARA RECOGER EL RESULTADO DE LA CONSULTA Y  MOSTRARLOS EN PANTALLA
    S_NOMPARLAMENT              DESPESES.NOMPARLAMENT%type;
    S_NOMBREPARLAMENT           PARLAMENTS.NOMBREPARLAMENTARIS%TYPE;
    S_NOMBREPARLAMENT_ANY       PARLAMENTS.NOMBREPARLAMENTARIS%TYPE;
    S_DESP_MITJA_PARLAMENTARIS  NUMBER;
    S_DESP_SUMA_PARLAMENTARIS   NUMBER;
    S_DESP_SUMA_PARLAMENT_SOL   NUMBER;
    S_DESP_SUMA_TOTAL           NUMBER;

    -- NO EXISTEN GASTOS PARA ESE AO
    EXC_NO_DESPESA              EXCEPTION;
    error_despesa               NUMBER;

    CURSOR_R             SYS_REFCURSOR;
    
Begin    
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_DespTotalTotsParlam';
    param_entrada := ('ANY:' ||V_ANY );


    SELECT COUNT (*) INTO ERROR_DESPESA 
    FROM DESPESES 
    WHERE EXTRACT(YEAR FROM DATA_DESPESA)=V_ANY;
    
    IF ERROR_DESPESA= 0 THEN RAISE EXC_NO_DESPESA; END IF;

    OPEN CURSOR_R FOR
    SELECT
       PARLAMENTS.NOMPARLAMENT                    AS PARLAMENT,
       PARLAMENTS.nombreparlamentaris             as PARLAMENTARIS_ACTUALS,
    
       -- Numero de parlamentarios de ese parlamento con gastos ese ao
       ( SELECT COUNT (*)  FROM(
            select  NOMPAR, CODIPAR 
                from 
                ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, 
                                   DESPESES2.CODIPARLAMENTARI as CODIPAR 
                       from DESPESES DESPESES2 where 
                       extract (year from DESPESES2.DATA_DESPESA)=V_ANY
                       and DESPESES2.TIPUSDESPESA='PARLAMENTARI'), PARLAMENTS
            where NOMPAR=PARLAMENTS.NOMPARLAMENT  
        )
        WHERE PARLAMENTS.NOMPARLAMENT=NOMPAR )    AS PARLAMENT_AMB_DESPESA,

       -- Calculo del ghasto medios de los parlamentarios de ese parlamento, en ese ao
       -- Se calculan como la suma de los gastos directos de los parlamentarios
       -- dividido por el numero de parlamentarios que para ese parlamento y ao, tienen algun gasto registrado
       ROUND  (  --gasto total de los parlamentarios de un parlamento y ao
                (SELECT SUM (DESPESES.IMPORT_DESPESA) 
                 FROM DESPESES 
                 WHERE PARLAMENTS.NOMPARLAMENT=DESPESES.NOMPARLAMENT AND   
                       DESPESES.TIPUSDESPESA='PARLAMENTARI'  AND 
                      ( EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY)
                 )/(  
                 -- Numero de parlamentarios de ese parlamento con gastos ese ao
                 SELECT COUNT (*)  FROM(
                    select  NOMPAR, CODIPAR 
                    from 
                    ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, 
                                      DESPESES2.CODIPARLAMENTARI as CODIPAR 
                      FROM DESPESES DESPESES2 WHERE 
                      extract (year from DESPESES2.DATA_DESPESA)=V_ANY
                      and DESPESES2.TIPUSDESPESA='PARLAMENTARI'), PARLAMENTS
                    where NOMPAR=PARLAMENTS.NOMPARLAMENT  
                    )
                WHERE PARLAMENTS.NOMPARLAMENT=NOMPAR
                )              
            ,2) AS MITJA_PARLAMENTARI,                                               
                                                        
     
       (SELECT SUM (DESPESES.IMPORT_DESPESA) 
              from DESPESES 
              WHERE PARLAMENTS.nomparlament=DESPESES.nomparlament AND   
              DESPESES.TIPUSDESPESA='PARLAMENTARI'  AND 
              ( exTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY)) AS NOMES_PARLAMENTARIs,
              
      (SELECT SUM (DESPESES.IMPORT_DESPESA) 
              from DESPESES  
              WHERE PARLAMENTS.nomparlament=DESPESES.nomparlament AND   
              DESPESES.TIPUSDESPESA='PARLAMENT'    AND 
              ( exTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY) ) AS NOMES_PARLAMENT,
    
      (SELECT SUM (DESPESES.IMPORT_DESPESA) 
              from DESPESES  
              WHERE PARLAMENTS.nomparlament=DESPESES.nomparlament AND   
              ( exTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY) ) AS DESPESA_TOTAL
              
       from PARLAMENTS PARLAMENTS
       GROUP BY    PARLAMENTS.nomparlament , PARLAMENTS.nombreparlamentaris;


       dbms_output.put_line ('***** GASTOS DE TODOS LOS PARLAMENTOS');
       dbms_output.put_line ('***** DURANTE EL AO: '|| V_ANY);
       
       DBMS_OUTPUT.PUT_LINE    (LPAD ('NOMBRE PARLAMENTO',25)||
                                LPAD ('PARLAMENTARIOS',15)||
                                LPAD ('PARLAMENTARIOS C/GASTO',30)||
                                LPAD ('MEDIA PARLAMENTARIOS',25)||
                                LPAD ('TOTAL PARLAMENTARIOS',25)||
                                LPAD ('TOTAL PARLAMENTO',25)||
                                LPAD ('TOTAL AMBOS', 15)); 
     
       LOOP FETCH CURSOR_R INTO S_NOMPARLAMENT,
                                S_NOMBREPARLAMENT,
                                S_NOMBREPARLAMENT_ANY,
                                S_DESP_MITJA_PARLAMENTARIS,
                                S_DESP_SUMA_PARLAMENTARIS,
                                S_DESP_SUMA_PARLAMENT_SOL,
                                S_DESP_SUMA_TOTAL;
       EXIT WHEN CURSOR_R%NOTFOUND;  
       DBMS_OUTPUT.PUT_LINE    (LPAD(S_NOMPARLAMENT,25)||
                                LPAD (S_NOMBREPARLAMENT||' ',15)||
                                LPAD (S_NOMBREPARLAMENT_ANY||' ',30)||
                                LPAD (S_DESP_MITJA_PARLAMENTARIS||' ',25)||
                                LPAD (S_DESP_SUMA_PARLAMENTARIS||' ',25)||
                                LPAD (S_DESP_SUMA_PARLAMENT_SOL||' ',25)||
                                LPAD (S_DESP_SUMA_TOTAL||' ', 15));
       end LOOP;

       -- registro el resultado de la ejecucion
       RSP:='OK';
       param_sortida := 'proc_DespTotalTotsParlam ejecutado correctamente';
       INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
       VALUES (SEQ_ID_LOG.NextVal,
               data_proces,
               nom_proces,
               param_entrada,
               param_sortida,
               rsp);
       COMMIT;


      ----------------
      -- EXCEPCIONES--
      ----------------

      EXCEPTION
        when exc_no_despesa then
        RSP:=('ERROR_PROC_C_01: no se registraron gastos en el ao '||V_ANY);
        INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
        VALUES (SEQ_ID_LOG.NextVal,
                data_proces,
                nom_proces,
                param_entrada,
                NULL,
                RSP); 
        commit;
      
        
      WHEN OTHERS THEN 
        --DBMS_OUTPUT.PUT_LINE(SQLCODE);
        --DBMS_OUTPUT.PUT_LINE(SQLERRM);
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      rsp);
      commit;
    
end proc_desptotaltotsparlam;


/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  BEGIN
      PROC_DESPTOTALTOTSPARLAM (2005 , RSP) ;
      ULTIMA_LINEA_LOG;
end;
*/
