-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_DespPerCategorias 
-- *** REFERENCIA              : D 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** Elabora, para un ao y parlamento determinados un listado
-- *** del gasto por categoras, incluyendo cdigo de la cat. del
-- *** gasto, descripcin de la categora, y suma del gasto
-- *** imputable a esa categora
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** AO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : Devuelve "OK" si el resultado es correcto
-- ***       Codigo y descripcion del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_D_01: El parlamento no existe
-- *** ERROR_PROC_D_02: No hay gastos para ese parlamento en el ao solicitado
-- ******************************************************************** 

CREATE OR REPLACE
PROCEDURE proc_DespPerCategorias (V_NOMPARLAMENT IN DESPESES.NOMPARLAMENT%TYPE,
                                  V_ANY IN NUMBER,
                                  RSP OUT NOCOPY CLOB) AS

    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;

    -- VARIBALES PARA RECOGER EL RESULTADO DE LA CONSULTA Y  MOSTRARLOS EN PANTALLA
    
    S_CODICATDESPESA               DESPESES.NOMPARLAMENT%type;
    S_DESCRIPCIODESPESA            TIPUSDESPESA.DESCRIPCIODESPESA%type;
    S_IMPORTTOTAL                 Number;

    -- NO EXISTE EL PARLAMENTO
    EXC_NO_PARLAMENT            EXCEPTION;
    error_parlament             NUMBER;
    
    -- NO HY GASTOS PARA ESE PARLAMENTO Y AO
    EXC_NO_DESPESA_PARLAMENT    EXCEPTION;
    error_despesa               NUMBER;

    CURSOR_R             SYS_REFCURSOR;
    
Begin    
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_desppercategorias';
    param_entrada := ('ANY:' ||V_ANY );

    -- NO EXSTE EL PARLAMENTO
    SELECT COUNT(*) INTO error_parlament FROM
    PARLAMENTS WHERE
    PARLAMENTS.NOMPARLAMENT=V_NOMPARLAMENT;

    IF ERROR_PARLAMENT=0  THEN RAISE EXC_NO_PARLAMENT; END IF;

    -- NO EXISTEN GASTOS PARA ESE PARLAMENTO EN EL AO SOLICITADO
    SELECT COUNT(*) INTO error_despesa 
    FROM PARLAMENTARIS   
       INNER JOIN DESPESES ON DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT
    WHERE  EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY AND DESPESES.NOMPARLAMENT=V_NOMPARLAMENT; 
    
    IF ERROR_DESPESA=0  THEN RAISE EXC_NO_DESPESA_PARLAMENT; END IF;   


    OPEN CURSOR_R FOR
    SELECT
       TIPUSDESPESA.CODICATDESPESA,
       TIPUSDESPESA.DESCRIPCIODESPESA,
       SUM (DESPESES.IMPORT_DESPESA)
    from TIPUSDESPESA
      INNER JOIN DESPESES ON DESPESES.CODICATDESPESA=TIPUSDESPESA.CODICATDESPESA and DESPESES.TIPUSDESPESA=TIPUSDESPESA.TIPUSDESPESA
    
    WHERE  (EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY ) and DESPESES.NOMPARLAMENT=V_NOMPARLAMENT
    group by    TIPUSDESPESA.CODICATDESPESA,   TIPUSDESPESA.DESCRIPCIODESPESA;



    DBMS_OUTPUT.PUT_LINE ('***** GASTOS POR CATEGORIAS');
    DBMS_OUTPUT.PUT_LINE ('***** DEL PARLAMENTO'||V_NOMPARLAMENT);    
    dbms_output.put_line ('***** DURANTE EL AO: '|| V_ANY);
    
    DBMS_OUTPUT.PUT_LINE    (LPAD ('CODIGO CAT. DEL GASTO',25)||
                             LPAD ('DESCRIPCION DEL GASTO',35)||
                             LPAD ('IMPORTE TOTAL',16)
                             ); 

    LOOP FETCH CURSOR_R INTO S_CODICATDESPESA,
                             S_DESCRIPCIODESPESA,
                             S_IMPORTTOTAL;
    EXIT WHEN CURSOR_R%NOTFOUND;  
    DBMS_OUTPUT.PUT_LINE    (LPAD(S_CODICATDESPESA,25)||
                             LPAD (S_DESCRIPCIODESPESA||' ',35)||
                             LPAD (S_IMPORTTOTAL||' ',16)
                             );
    end LOOP;

    -- registro el resultado de la ejecucion
    RSP:='OK';
    param_sortida := 'proc_DespPerCategorias ejecutado correctamente';
    INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
    VALUES (SEQ_ID_LOG.NextVal,
            data_proces,
            nom_proces,
            param_entrada,
            param_sortida,
            rsp);
    COMMIT;


      ----------------
      -- EXCEPCIONES--
      ----------------
    
    EXCEPTION
      when exc_no_parlament then
      RSP:=('ERROR_PROC_D_01: El parlamento:  '||V_NOMPARLAMENT||' no existe');
      INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
      VALUES (SEQ_ID_LOG.NextVal,
              data_proces,
              nom_proces,
              param_entrada,
              NULL,
              RSP); 
      commit;
    
      when EXC_NO_DESPESA_PARLAMENT then
      RSP:=('ERROR_PROC_D_02: El parlamento: '||V_NOMPARLAMENT||' no tiene gastos en el ao :'|| V_ANY); 
      INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
      VALUES (SEQ_ID_LOG.NextVal,
              data_proces,
              nom_proces,
              param_entrada,
              NULL,
              RSP); 
      commit;
      
      WHEN OTHERS THEN 
        --DBMS_OUTPUT.PUT_LINE(SQLCODE);
        --DBMS_OUTPUT.PUT_LINE(SQLERRM);
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      rsp);
      commit;
    
end PROC_DESPPERCATEGORIAS;

/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  begin
      proc_DespPerCategorias ('ASSEMBLEE NATIONALE',2013,RSP);
      ULTIMA_LINEA_LOG;
end;
*/
