-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_ParlamDespejaSupAMitjana 
-- *** REFERENCIA              : E 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** Elabora, para un ao y parlamento determinados, un listado
-- *** de los parlamentarios que superan la media del gasto de
-- *** los parlamentarios para ese parlamento 
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** PARLAMENTO: Texto
-- *** AO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : Devuelve "OK" si el resultado es correcto
-- ***       Codigo y descripcion del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_E_01: El parlamento no existe
-- *** ERROR_PROC_E_02: No hay gastos para ese parlamento en el ao solicitado
-- ******************************************************************** 

CREATE OR REPLACE
procedure proc_ParlamDespejaSupAMitjana (V_NOMPARLAMENT in DESPESES.NOMPARLAMENT%type,
                                  V_ANY IN NUMBER,
                                  RSP OUT NOCOPY CLOB) AS


    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;


   desptotalanyparlamentari   NUMBER;
   numparlamentsambdespesa    NUMBER;
   
   
    -- VARIBALES PARA RECOGER EL RESULTADO DE LA CONSULTA Y  MOSTRARLOS EN PANTALLA
   
    S_NOMPARLAMENT                DESPESES.NOMPARLAMENT%type;
    S_CODIPARLAMENTARI            DESPESES.CODIPARLAMENTARI%TYPE;
    S_PARTITPOL                   DESPESES.PARTITPOL%TYPE;
    S_NOM                         PARLAMENTARIS.NOM%TYPE;
    S_COGNOMS                     PARLAMENTARIS.COGNOMS%TYPE;
    S_SUMA_TOTAL_PARLAMENT        NUMBER;
    S_SUMA_PARLAMENTARI           NUMBER;
    S_NUMPARLAMAMBDESPESA         NUMBER;
    S_MITJA_PARLAMENTARIS         NUMBER;
    S_PERCET_SUPERA               NUMBER;
  
    -- NO EXISTE EL PARLAMENTO
    EXC_NO_PARLAMENT            EXCEPTION;
    error_parlament             NUMBER;
    
    -- NO HY GASTOS PARA ESE PARLAMENTO Y AO
    EXC_NO_DESPESA_PARLAMENT    EXCEPTION;
    error_despesa               NUMBER;

    CURSOR_R             SYS_REFCURSOR;
    
Begin    
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_ParlamDespejaSupAMitjana';
    param_entrada := ('ANY:' ||V_ANY );


    -- NO EXSTE EL PARLAMENTO
    SELECT COUNT(*) INTO error_parlament FROM
    PARLAMENTS WHERE
    PARLAMENTS.NOMPARLAMENT=V_NOMPARLAMENT;

    IF ERROR_PARLAMENT=0  THEN RAISE EXC_NO_PARLAMENT; END IF;

    -- NO EXISTEN GASTOS PARA ESE PARLAMENTO EN EL AO SOLICITADO
    SELECT COUNT(*) INTO error_despesa 
    FROM PARLAMENTARIS   
       INNER JOIN DESPESES ON DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT
    WHERE  EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY AND DESPESES.NOMPARLAMENT=V_NOMPARLAMENT; 
    
    IF ERROR_DESPESA=0  THEN RAISE EXC_NO_DESPESA_PARLAMENT; END IF;   


    -- GASTO TOTAL ASOCIADO A LOS PARLAMENTARIPS PARA ESE PARLAMENTO Y AO
    SELECT SUM (DESPESES.IMPORT_DESPESA) into desptotalanyparlamentari
    FROM DESPESES 
    WHERE 
       DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND
       DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
       EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)=V_ANY;
       
    -- NUMERO DE PARLAMENTARIOS CON GASTOS ESE AO
   SELECT COUNT (*) INTO numparlamentsambdespesa
   FROM
     (SELECT DISTINCT DESPESES.NOMPARLAMENT, DESPESES.CODIPARLAMENTARI
     FROM   DESPESES
   WHERE  
     DESPESES.NOMPARLAMENT=V_NOMPARLAMENT  AND
     DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
     EXTRACT (YEAR FROM DESPESES.DATA_DESPESA)=V_ANY);
  

    --******************************************************************************************
    --******************************************************************************************
    --********* SENTENCIA PARA CALCULAR PARLAMENTARIOS CON GASTTO SUP. A LA MEDIA DEL PARLAMENTO
    --******************************************************************************************
    --******************************************************************************************     
    OPEN CURSOR_R FOR 
    SELECT
         DESPESES.NOMPARLAMENT,
         DESPESES.CODIPARLAMENTARI,
         PARLAMENTARIS.PARTITPOL,
         PARLAMENTARIS.NOM,
         PARLAMENTARIS.COGNOMS,
         
         --***********************************************************************************     
         -- Suma de los gastos del parlamento asociados a parlamentarios, en el ao contemplado
         --***********************************************************************************     
         (SELECT SUM (DESPESES.IMPORT_DESPESA)
         FROM DESPESES 
               WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND 
               DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
               EXTRACT (YEAR FROM DATA_DESPESA)=V_ANY
         GROUP BY DESPESES.NOMPARLAMENT) AS SUMA_TOTAL_PARLAMENT,
         
         --***********************************************************************************      
         -- Suma de los gastos PROPIOS DE CADA PARLAMENTARIO, en el ao contemplado
         --***********************************************************************************      
         (SELECT SUM (DESPESES2.IMPORT_DESPESA)
         FROM DESPESES DESPESES2
               WHERE DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT AND 
               DESPESES2.TIPUSDESPESA='PARLAMENTARI' AND
               DESPESES2.NOMPARLAMENT=DESPESES.NOMPARLAMENT AND
               DESPESES2.CODIPARLAMENTARI=DESPESES.CODIPARLAMENTARI AND
               EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY 
         GROUP BY DESPESES2.NOMPARLAMENT, DESPESES2.CODIPARLAMENTARI) AS SUMA_PARLAMENTARI,
         
         --***********************************************************************************
         -- Numero de parlamentarios con gastos asociados para el parlamento y ao contemplados
         --***********************************************************************************     
        (SELECT COUNT (*)  FROM -- Este fragmento me cuenta las filas devueltas por la select mas interna
            (
            select  CODIPAR 
                FROM 
                -- Este fragmento me devuelve tantas filas como parlamentarios con gasto para el parlamento y ao contemplados
                ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, DESPESES2.CODIPARLAMENTARI as CODIPAR 
                       FROM DESPESES DESPESES2 WHERE 
                       EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY AND
                       DESPESES2.TIPUSDESPESA='PARLAMENTARI' and
                       DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT
                       )
         )) AS NumParlamAmbDespesa,
            
         --***********************************************************************************
         -- Media del gasto por parlamentario, para ese parlamento y ao, teniendo en cuenta 
         -- SOLAMENTE los parlamentarios que tienen registrado algun gasto
         --***********************************************************************************     
         ROUND ( 
         ( 
           (      -- Suma de los gastos del parlamento asociados a parlamentarios, en el ao contemplado
             SELECT SUM (DESPESES.IMPORT_DESPESA)
             FROM DESPESES 
                   WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND 
                   DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
                   EXTRACT (YEAR FROM DATA_DESPESA)=V_ANY
             GROUP BY DESPESES.NOMPARLAMENT
            )/(   -- Numero de parlamentarios con gastos asociados para el parlamento y ao contemplados
          SELECT COUNT (*)  FROM(
              select  CODIPAR 
                  from 
                  ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, 
                                     DESPESES2.CODIPARLAMENTARI as CODIPAR 
                         FROM DESPESES DESPESES2 WHERE 
                         extract (year from DESPESES2.DATA_DESPESA)=V_ANY
                         AND DESPESES2.TIPUSDESPESA='PARLAMENTARI')
              WHERE NOMPAR=V_NOMPARLAMENT)) 
          ),2) AS MITJA_parlamentaris,
        
         --***********************************************************************************
         -- PORCENTAJE EN EL QUE LOS GASTOS DE ESE PARLAMENTARIO, SUPERAN LA MEDIA 
         -- Se calcula como:
         -- ( Gastos de ese perlamentario * 100 / Media del gasto )- 100
         --***********************************************************************************     
          Round(
              --***********************************************************************************      
              -- Suma de los gastos PROPIOS DE CADA PARLAMENTARIO, en el ao contemplado
              --*********************************************************************************** 
            (SELECT SUM (DESPESES2.IMPORT_DESPESA)
            FROM DESPESES DESPESES2
                  WHERE DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT AND 
                  DESPESES2.TIPUSDESPESA='PARLAMENTARI' AND
                  DESPESES2.NOMPARLAMENT=DESPESES.NOMPARLAMENT AND
                  DESPESES2.CODIPARLAMENTARI=DESPESES.CODIPARLAMENTARI AND
                  EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY 
            GROUP BY DESPESES2.NOMPARLAMENT, DESPESES2.CODIPARLAMENTARI) *100/( 
               --***********************************************************************************
               -- Media del gasto por parlamentario, para ese parlamento y ao, teniendo en cuenta 
               -- SOLAMENTE los parlamentarios que tienen registrado algun gasto
               --***********************************************************************************    
            (       -- Suma de los gastos del parlamento asociados a parlamentarios, en el ao contemplado
              SELECT SUM (DESPESES.IMPORT_DESPESA)
              FROM DESPESES 
                    WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND 
                    DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
                    EXTRACT (YEAR FROM DATA_DESPESA)=V_ANY
              GROUP BY DESPESES.NOMPARLAMENT
             )/(
             SELECT COUNT (*)  FROM 
                 (    -- Este fragmento me cuenta las filas devueltas por la select mas interna
                 select  CODIPAR 
                     FROM 
                     -- Este fragmento me devuelve tantas filas como parlamentarios con gasto para el parlamento y ao contemplados
                     ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, DESPESES2.CODIPARLAMENTARI as CODIPAR 
                            FROM DESPESES DESPESES2 WHERE 
                            EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY AND
                            DESPESES2.TIPUSDESPESA='PARLAMENTARI' and
                            DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT
                      )
                 )  
              ) 
             )
           ,2) -100 AS PERCET_SUPERA
    
        from DESPESES
        
        -- Join para ir a buscar el nombre y apellidos del parlamentario
        INNER JOIN  PARLAMENTARIS  ON ( DESPESES.CODIPARLAMENTARI=PARLAMENTARIS.CODIPARLAMENTARI AND  DESPESES.NOMPARLAMENT=PARLAMENTARIS.NOMPARLAMENT)
        
        WHERE 
           EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY AND DESPESES.NOMPARLAMENT=V_NOMPARLAMENT
           GROUP BY      DESPESES.NOMPARLAMENT,
                         DESPESES.CODIPARLAMENTARI,
                         PARLAMENTARIS.PARTITPOL,
                         --PARLAMENTS.NOMBREPARLAMENTARIS, 
                         PARLAMENTARIS.NOM, 
                         PARLAMENTARIS.COGNOMS
        
        HAVING 
         --***********************************************************************************
         -- RESTRICCION DE LOS RESULTADOS DEVUELTOS: Solo devolvemos
         -- los parlamentarios CUYO GASTO ha sido MAYOR QUE la media de ese parlamento para ese ao
         --***********************************************************************************
         
              --***********************************************************************************      
              -- Suma de los gastos PROPIOS DE CADA PARLAMENTARIO, en el ao contemplado
              --*********************************************************************************** 
             (SELECT SUM (DESPESES2.IMPORT_DESPESA)
             FROM DESPESES DESPESES2
                   WHERE DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT AND 
                   DESPESES2.TIPUSDESPESA='PARLAMENTARI' AND
                   DESPESES2.NOMPARLAMENT=DESPESES.NOMPARLAMENT AND
                   DESPESES2.CODIPARLAMENTARI=DESPESES.CODIPARLAMENTARI AND
                   EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY 
             GROUP BY DESPESES2.NOMPARLAMENT, DESPESES2.CODIPARLAMENTARI) 
             >
             --***********************************************************************************
             -- Media del gasto por parlamentario, para ese parlamento y ao, teniendo en cuenta 
             -- SOLAMENTE los parlamentarios que tienen registrado algun gasto
             --***********************************************************************************   
             ROUND ( 
             ( 
             (       -- Suma de los gastos del parlamento asociados a parlamentarios, en el ao contemplado
               SELECT SUM (DESPESES.IMPORT_DESPESA)
               FROM DESPESES 
                     WHERE DESPESES.NOMPARLAMENT=V_NOMPARLAMENT AND 
                     DESPESES.TIPUSDESPESA='PARLAMENTARI' AND
                     EXTRACT (YEAR FROM DATA_DESPESA)=V_ANY
               GROUP BY DESPESES.NOMPARLAMENT
              )/(  -- Numero de parlamentarios con gastos asociados para el parlamento y ao contemplados
              SELECT COUNT (*)  FROM 
                  (    -- Este fragmento me cuenta las filas devueltas por la select mas interna
                  select  CODIPAR 
                      FROM 
                      -- Este fragmento me devuelve tantas filas como parlamentarios con gasto para el parlamento y ao contemplados
                      ( select distinct DESPESES2.NOMPARLAMENT as NOMPAR, DESPESES2.CODIPARLAMENTARI as CODIPAR 
                             FROM DESPESES DESPESES2 WHERE 
                             EXTRACT (YEAR FROM DESPESES2.DATA_DESPESA)=V_ANY AND
                             DESPESES2.TIPUSDESPESA='PARLAMENTARI' and
                             DESPESES2.NOMPARLAMENT=V_NOMPARLAMENT
                       )
                  )  
               ) 
              ),2);
              
    --******************************************************************************************
    --******************************************************************************************
    --***** FIN SENTENCIA PARA CALCULAR PARLAMENTARIOS CON GASTTO SUP. A LA MEDIA DEL PARLAMENTO
    --******************************************************************************************
    --******************************************************************************************                 


    -- PRESENTACION DE LOS RESULTADOS
    
    DBMS_OUTPUT.PUT_LINE ('***** PARLAMENTARIOS CON GASTO SUP A LA MEDIA');
    DBMS_OUTPUT.PUT_LINE ('***** DEL PARLAMENTO: '||V_NOMPARLAMENT);    
    dbms_output.put_line ('***** DURANTE EL AO: '|| V_ANY);
    dbms_output.put_line ('**********************************************');    
    DBMS_OUTPUT.PUT_LINE ('***** GASTO TOTAL DE LOS PARLAMENTARIOS  : '|| DESPTOTALANYPARLAMENTARI);
    dbms_output.put_line ('***** NUMERO DE PARLAMENTARIOS CON GASTO : '|| numparlamentsambdespesa);
    dbms_output.put_line ('***** MEDIA                              : '|| Round(desptotalanyparlamentari/numparlamentsambdespesa,2));

    DBMS_OUTPUT.PUT_LINE    (LPAD ('COD. PARLAMENT.',15)||
                             LPAD ('NOMBRE',20)||
                             LPAD ('APELLIDOS',30)||
                             LPAD ('PARTIDO POLITICO',30)||
                             LPAD ('GASTOS DIRECTOS',18)||  
                             LPAD ('PORCENT. SUPERA',18)
                             ); 

          
    LOOP FETCH CURSOR_R INTO 
       S_NOMPARLAMENT,         
       S_CODIPARLAMENTARI,       
       S_PARTITPOL,              
       S_NOM,                   
       S_COGNOMS,                 
       S_SUMA_TOTAL_PARLAMENT,     
       S_SUMA_PARLAMENTARI,       
       S_NUMPARLAMAMBDESPESA,      
       S_MITJA_PARLAMENTARIS,       
       S_PERCET_SUPERA;         
       EXIT WHEN CURSOR_R%NOTFOUND;  
       
       DBMS_OUTPUT.PUT_LINE (LPAD (S_CODIPARLAMENTARI,15)||
                             LPAD (S_NOM ,20)||
                             LPAD (S_COGNOMS ,30)||
                             LPAD (S_PARTITPOL  ,30)||
                             LPAD (S_SUMA_PARLAMENTARI,18)||  
                             LPAD (S_PERCET_SUPERA,18)
                             ); 

    END LOOP;

    -- registro el resultado de la ejecucion
    RSP:='OK';
    param_sortida := 'proc_ParlamDespejaSupAMitjana ejecutado correctamente';
    INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
    VALUES (SEQ_ID_LOG.NextVal,
            data_proces,
            nom_proces,
            param_entrada,
            param_sortida,
            rsp);
    COMMIT;


      ----------------
      -- EXCEPCIONES--
      ----------------
    
    EXCEPTION
      when EXC_NO_PARLAMENT then
      RSP:=('ERROR_PROC_E_01: El parlamento:  '||V_NOMPARLAMENT||' no existe');
      INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
      VALUES (SEQ_ID_LOG.NextVal,
              data_proces,
              nom_proces,
              param_entrada,
              NULL,
              RSP); 
      commit;
    
      when EXC_NO_DESPESA_PARLAMENT then
      RSP:=('ERROR_PROC_E_02: El parlamento: '||V_NOMPARLAMENT||' no tiene gastos en el ao :'|| V_ANY); 
      INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
      VALUES (SEQ_ID_LOG.NextVal,
              data_proces,
              nom_proces,
              param_entrada,
              NULL,
              RSP); 
      commit;
      
      
      WHEN OTHERS THEN 
        DBMS_OUTPUT.PUT_LINE(SQLCODE);
        DBMS_OUTPUT.PUT_LINE(SQLERRM);
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO LOG_PROCEDIMENTS
              VALUES (SEQ_ID_LOG.NextVal,
                      data_proces,
                      nom_proces,
                      param_entrada,
                      NULL,
                      rsp);
      commit;
  
END proc_ParlamDespejaSupAMitjana;

/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  begin
      PROC_PARLAMDESPEJASUPAMITJANA ('DEUSTCHER BUNDESTAG',2005, RSP);
      ULTIMA_LINEA_LOG;
end;
*/
