-- ******************************************************************** 
-- *** NOMBRE DEL PROCEDIMIENTO: proc_EstatContableParlaments 
-- *** REFERENCIA              : F 
-- ******************************************************************** 
-- *** DESCRIPCION *** 
-- *** elabora un listado que representa el gasto contable de
-- *** los parlamentos, incluyendo: Parlamento, nmero de
-- *** parlamentarios, suma de cada uno de los siguientes
-- *** conceptos: gasto pendiente de aprobar, gasto aprobado,
-- *** gasto pendiente de abonar, gasto abonado
-- ******************************************************************** 
-- *** PARAMETROS DE ENTRADA *** 
-- *** 
-- *** AO: Numero
-- ******************************************************************** 
-- *** PARAMETROS DE SALIDA *** 
-- *** 
-- *** RSP : Devuelve "OK" si el resultado es correcto
-- ***       Codigo y descripcion del error en caso contrario
-- ******************************************************************** 
-- *** CODIGOS DE ERROR *** 
-- *** 
-- *** ERROR_PROC_F_01: No se registraron gastos en el ao solicitado
-- ******************************************************************** 

CREATE OR REPLACE
PROCEDURE proc_EstatContableParlaments (V_ANY IN NUMBER, RSP OUT NOCOPY CLOB) AS

-------------------------------------------------------------------   
-- Declaraci de variables                                       --
-------------------------------------------------------------------
    data_proces          VARCHAR2(20);
    nom_proces           VARCHAR2(50);
    param_entrada        CLOB;
    param_sortida        CLOB;
        

    S_NOMPARLAMENT       DESPESES.NOMPARLAMENT%TYPE;
    S_NOMBREPARLAM       PARLAMENTS.NOMBREPARLAMENTARIS%TYPE;
    S_PENDENT_APROVA     NUMBER;
    S_APROVADA           NUMBER;
    S_PENDENT_ABONA      NUMBER;
    S_ABONADA            NUMBER;


    CURSOR_R             SYS_REFCURSOR;
-------------------------------------------------------------------    
-- EXCEPCIOMES                                      --
-------------------------------------------------------------------

    -- NO EXISTEN GASTOS PARA ESE AO
    EXC_NO_DESPESA              EXCEPTION;
    error_despesa               NUMBER;
    

BEGIN
           
    data_proces:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
    nom_proces:='proc_EstatContableParlaments';
    param_entrada := ('ANY:'||V_ANY );
    
    
    SELECT COUNT (*) INTO ERROR_DESPESA 
    FROM DESPESES 
    WHERE EXTRACT(YEAR FROM DATA_DESPESA)=V_ANY;
    IF ERROR_DESPESA= 0 THEN RAISE EXC_NO_DESPESA; END IF;
    
    
    OPEN CURSOR_R FOR
    SELECT 
       NOMPARLAMENT,
       NOMBREPARLAMENTARIS,
       sum(PENDENT_APROVACIO),
       sum(APROVADA), 
       sum (PENDENT_ABONAMENT),
       sum (ABONADA) from 
       (
       SELECT
        PARLAMENTS.NOMPARLAMENT,
        PARLAMENTS.NOMBREPARLAMENTARIS ,
       
        ESTATDESPESA.DESCRIPESTAT,
        -- SUM (DESPESAS.IMPORT_DESPESA)   as SUMA_DESPESA
        SUM  ( case when  DESPESES.CODIESTAT=1 THEN (DESPESES.IMPORT_DESPESA) ELSE 0 END) as PENDENT_APROVACIO,
        SUM  ( case when  DESPESES.CODIESTAT=2 THEN (DESPESES.IMPORT_DESPESA) ELSE 0 END) as APROVADA,
        SUM  ( case when  DESPESES.CODIESTAT=3 THEN (DESPESES.IMPORT_DESPESA) ELSE 0 END) as PENDENT_ABONAMENT,
        SUM  ( CASE WHEN  DESPESES.CODIESTAT=4 THEN (DESPESES.IMPORT_DESPESA) ELSE 0 END) AS ABONADA 
       from DESPESES 
       INNER JOIN ESTATDESPESA ON DESPESES.CODIESTAT=ESTATDESPESA.CODIESTAT
       INNER JOIN PARLAMENTS ON  DESPESES.NOMPARLAMENT=PARLAMENTS.NOMPARLAMENT 
       WHERE  EXTRACT(YEAR FROM DESPESES.DATA_DESPESA)=V_ANY or ( DESPESES.DATA_DESPESA is null )
       GROUP BY   PARLAMENTS.NOMPARLAMENT , PARLAMENTS.NOMBREPARLAMENTARIS, ESTATDESPESA.DESCRIPESTAT
       
       )
    GROUP BY  NOMPARLAMENT,NOMBREPARLAMENTARIS ;
    
    
    
    dbms_output.put_line ('***** ESTADO CONTABLE DE TODOS LOS PARLAMENTOS: ' );
    dbms_output.put_line ('***** EN EL AO: '|| V_ANY);

    DBMS_OUTPUT.PUT_LINE    (LPAD ('PARLAMENTO',25)||
                             LPAD ('NUM. PARLAMENTARIOS',20)||
                             LPAD ('PEND. APROBAR',15)||
                             LPAD ('APROBADAS',15)||
                             LPAD ('PEND. ABONAR',15)||  
                             LPAD ('ABONADAS',15)
                             ); 
                             
    LOOP FETCH CURSOR_R INTO 
                   S_NOMPARLAMENT,
                   S_NOMBREPARLAM,
                   S_PENDENT_APROVA,
                   S_APROVADA,
                   S_PENDENT_ABONA,
                   S_ABONADA;    

    EXIT WHEN CURSOR_R%NOTFOUND;  
       DBMS_OUTPUT.PUT_LINE (LPAD (S_NOMPARLAMENT,25)||
                             LPAD (S_NOMBREPARLAM ,20)||
                             LPAD (S_PENDENT_APROVA ,15)||
                             LPAD (S_APROVADA  ,15)||
                             LPAD (S_PENDENT_ABONA,15)||  
                             LPAD (S_ABONADA,15)
                             ); 
    
    END LOOP;
    
    RSP:='OK';
    param_sortida := 'proc_EstatContableParlaments ejecutado correctamente';
    INSERT INTO LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
    VALUES (SEQ_ID_LOG.NextVal,
          data_proces,
          nom_proces,
          param_entrada,
          param_sortida,
          RSP);
    COMMIT;
    
    ----------------
    -- EXCEPCIONES--
    ----------------
    
    EXCEPTION
    WHEN EXC_NO_DESPESA THEN
    RSP:=('ERROR_PROC_F_01: no se registraron gastos en el ao '||V_ANY);
    INSERT INTO  LOG_PROCEDIMENTS ("ID_LOG","DATA_PROCES","PROC_EXECUTAT","PARAM_ENTRADA","PARAM_SORTIDA","RESULTAT_EXEC")
    VALUES (SEQ_ID_LOG.NextVal,
            data_proces,
            nom_proces,
            param_entrada,
            NULL,
            RSP); 
    commit;
    
    
    WHEN OTHERS THEN 
      DBMS_OUTPUT.PUT_LINE(SQLCODE);
      DBMS_OUTPUT.PUT_LINE(SQLERRM);
      RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
      INSERT INTO LOG_PROCEDIMENTS
            VALUES (SEQ_ID_LOG.NextVal,
                    data_proces,
                    nom_proces,
                    param_entrada,
                    NULL,
                    rsp);
    commit;
END proc_EstatContableParlaments ;


/*
SET SERVEROUTPUT ON SIZE UNLIMITED     
SET DEFINE OFF

DECLARE 
  RSP    CLOB;
  BEGIN
      proc_EstatContableParlaments (2013, RSP);
      ULTIMA_LINEA_LOG;
end;
*/
